# Copyright 2005 Divmod, Inc.  See LICENSE file for details

import itertools

from OpenSSL import SSL
from OpenSSL.crypto import PKey, X509, X509Req
from OpenSSL.crypto import TYPE_RSA

from twisted.trial import unittest, util
from twisted.internet import protocol, defer, reactor
from twisted.python import log

from vertex import sslverify

counter = itertools.count().next
def makeCertificate(**kw):
    keypair = PKey()
    keypair.generate_key(TYPE_RSA, 1024)

    certificate = X509()
    certificate.gmtime_adj_notBefore(0)
    certificate.gmtime_adj_notAfter(60 * 60 * 24 * 365) # One year
    for xname in certificate.get_issuer(), certificate.get_subject():
        for (k, v) in kw.items():
            setattr(xname, k, v)

    certificate.set_serial_number(counter())
    certificate.set_pubkey(keypair)
    certificate.sign(keypair, "md5")

    return keypair, certificate

def otherMakeCertificate(**kw):
    keypair = PKey()
    keypair.generate_key(TYPE_RSA, 1024)

    req = X509Req()
    subj = req.get_subject()
    for (k, v) in kw.items():
        setattr(subj, k, v)

    req.set_pubkey(keypair)
    req.sign(keypair, "md5")

    cert = X509()
    cert.set_serial_number(counter())
    cert.gmtime_adj_notBefore(0)
    cert.gmtime_adj_notAfter(60 * 60 * 24 * 365) # One year

    cert.set_issuer(req.get_subject())
    cert.set_subject(req.get_subject())
    cert.set_pubkey(req.get_pubkey())
    cert.sign(keypair, "md5")

    return keypair, cert


class DataCallbackProtocol(protocol.Protocol):
    def dataReceived(self, data):
        d, self.factory.onData = self.factory.onData, None
        if d is not None:
            d.callback(data)

    def connectionLost(self, reason):
        d, self.factory.onLost = self.factory.onLost, None
        if d is not None:
            d.errback(reason)

class WritingProtocol(protocol.Protocol):
    byte = 'x'
    def connectionMade(self):
        self.transport.write(self.byte)

    def connectionLost(self, reason):
        self.factory.onLost.errback(reason)


class OpenSSLOptions(unittest.TestCase):
    serverPort = clientConn = None
    onServerLost = onClientLost = None

    def setUpClass(self):
        self.sKey, self.sCert = makeCertificate(
            O="Server Test Certificate",
            CN="server")
        self.cKey, self.cCert = makeCertificate(
            O="Client Test Certificate",
            CN="client")

    def tearDown(self):
        if self.serverPort is not None:
            self.serverPort.stopListening()
        if self.clientConn is not None:
            self.clientConn.disconnect()

        L = []
        if self.onServerLost is not None:
            L.append(self.onServerLost)
        if self.onClientLost is not None:
            L.append(self.onClientLost)

        util.wait(defer.DeferredList(L, consumeErrors=True))

    def loopback(self, serverCertOpts, clientCertOpts,
                 onServerLost=None, onClientLost=None, onData=None):
        if onServerLost is None:
            self.onServerLost = onServerLost = defer.Deferred()
        if onClientLost is None:
            self.onClientLost = onClientLost = defer.Deferred()
        if onData is None:
            onData = defer.Deferred()

        serverFactory = protocol.ServerFactory()
        serverFactory.protocol = DataCallbackProtocol
        serverFactory.onLost = onServerLost
        serverFactory.onData = onData

        clientFactory = protocol.ClientFactory()
        clientFactory.protocol = WritingProtocol
        clientFactory.onLost = onClientLost

        self.serverPort = reactor.listenSSL(0, serverFactory, serverCertOpts)
        self.clientConn = reactor.connectSSL('127.0.0.1', self.serverPort.getHost().port,
                                             clientFactory, clientCertOpts)

    def testAbbreviatingDistinguishedNames(self):
        self.assertEquals(sslverify.DN(CN='a', OU='hello'),
                          sslverify.DistinguishedName(commonName='a', organizationalUnitName='hello'))
        self.assertNotEquals(sslverify.DN(CN='a', OU='hello'),
                             sslverify.DN(CN='a', OU='hello', emailAddress='xxx'))
        dn = sslverify.DN(CN='abcdefg')
        self.assertRaises(AttributeError, setattr, dn, 'Cn', 'x')
        self.assertEquals(dn.CN, dn.commonName)
        dn.CN = 'bcdefga'
        self.assertEquals(dn.CN, dn.commonName)

    def testAllowedAnonymousClientConnection(self):
        onData = defer.Deferred()
        self.loopback(sslverify.OpenSSLCertificateOptions(privateKey=self.sKey, certificate=self.sCert, requireCertificate=False),
                      sslverify.OpenSSLCertificateOptions(requireCertificate=False),
                      onData=onData)

        result = util.wait(onData)
        self.assertEquals(result, WritingProtocol.byte)

    def testRefusedAnonymousClientConnection(self):
        onServerLost = defer.Deferred()
        onClientLost = defer.Deferred()
        self.loopback(sslverify.OpenSSLCertificateOptions(privateKey=self.sKey, certificate=self.sCert, verify=True, caCerts=[self.sCert], requireCertificate=True),
                      sslverify.OpenSSLCertificateOptions(requireCertificate=False),
                      onServerLost=onServerLost,
                      onClientLost=onClientLost)

        d = defer.DeferredList([onClientLost, onServerLost], consumeErrors=True)
        (cSuccess, cResult), (sSuccess, sResult) = util.deferredResult(d)

        self.failIf(cSuccess)
        self.failIf(sSuccess)

        # XXX Twisted doesn't report SSL errors as SSL errors, but in the
        # future it will.

        # cResult.trap(SSL.Error)
        # sResult.trap(SSL.Error)

        # XXX So instead for now, we flush errors that got logged so the test
        # can actually pass.
        errors = log.flushErrors(SSL.Error)
        self.assertEquals(len(errors), 2)

    def testFailedCertificateVerification(self):
        onServerLost = defer.Deferred()
        onClientLost = defer.Deferred()
        self.loopback(sslverify.OpenSSLCertificateOptions(privateKey=self.sKey, certificate=self.sCert, verify=False, requireCertificate=False),
                      sslverify.OpenSSLCertificateOptions(verify=True, requireCertificate=False, caCerts=[self.cCert]),
                      onServerLost=onServerLost,
                      onClientLost=onClientLost)

        d = defer.DeferredList([onClientLost, onServerLost], consumeErrors=True)
        (cSuccess, cResult), (sSuccess, sResult) = util.deferredResult(d)

        self.failIf(cSuccess)
        self.failIf(sSuccess)
        errors = log.flushErrors(SSL.Error)
        self.assertEquals(len(errors), 2)

    def testSuccessfulCertificateVerification(self):
        onData = defer.Deferred()
        self.loopback(sslverify.OpenSSLCertificateOptions(privateKey=self.sKey, certificate=self.sCert, verify=False, requireCertificate=False),
                      sslverify.OpenSSLCertificateOptions(verify=True, requireCertificate=True, caCerts=[self.sCert]),
                      onData=onData)

        result = util.deferredResult(onData)
        self.assertEquals(result, WritingProtocol.byte)

    def testSuccessfulSymmetricSelfSignedCertificateVerification(self):
        onData = defer.Deferred()
        self.loopback(sslverify.OpenSSLCertificateOptions(privateKey=self.sKey, certificate=self.sCert, verify=True, requireCertificate=True, caCerts=[self.cCert]),
                      sslverify.OpenSSLCertificateOptions(privateKey=self.cKey, certificate=self.cCert, verify=True, requireCertificate=True, caCerts=[self.sCert]),
                      onData=onData)

        result = util.deferredResult(onData)
        self.assertEquals(result, WritingProtocol.byte)

    def testVerification(self):
        clientDN = sslverify.DistinguishedName(commonName='client')
        clientKey = sslverify.KeyPair.generate()
        clientCertReq = clientKey.certificateRequest(clientDN)

        serverDN = sslverify.DistinguishedName(commonName='server')
        serverKey = sslverify.KeyPair.generate()
        serverCertReq = serverKey.certificateRequest(serverDN)

        ##
        clientSelfCertReq = clientKey.certificateRequest(clientDN)
        clientSelfCertData = clientKey.signCertificateRequest(clientDN, clientSelfCertReq,
                                                              lambda dn: True,
                                                              132)
        clientSelfCert = clientKey.newCertificate(clientSelfCertData)
        ##

        ##
        serverSelfCertReq = serverKey.certificateRequest(serverDN)
        serverSelfCertData = serverKey.signCertificateRequest(serverDN, serverSelfCertReq,
                                                              lambda dn: True,
                                                              516)
        serverSelfCert = serverKey.newCertificate(serverSelfCertData)
        ##

        ##
        clientCertData = serverKey.signCertificateRequest(serverDN, clientCertReq,
                                                          lambda dn: True,
                                                          7)
        clientCert = clientKey.newCertificate(clientCertData)
        ##

        ##
        serverCertData = clientKey.signCertificateRequest(clientDN, serverCertReq,
                                                          lambda dn: True,
                                                          42)
        serverCert = serverKey.newCertificate(serverCertData)
        ##

        onData = defer.Deferred()

        serverOpts = serverCert.options(serverSelfCert)
        clientOpts = clientCert.options(clientSelfCert)

        self.loopback(serverOpts,
                      clientOpts,
                      onData=onData)

        result = util.deferredResult(onData)
        self.assertEquals(result, WritingProtocol.byte)



# Copyright 2005 Divmod, Inc.  See LICENSE file for details
# -*- test-case-name: vertex.test.test_juice -*-

__metaclass__ = type

from twisted.internet.protocol import Protocol

class LineReceiver(Protocol):

    lineMode = True
    MAX_LINE_LENGTH = 1024 * 1024
    buffer = ''
    delimiter = '\r\n'

    def lineReceived(self, line):
        pass

    def rawDataReceived(self, data):
        pass

    def setLineMode(self, extra=''):
        self.lineMode = True
        if extra:
            self.dataReceived(extra)

    def isDisconnecting(self):
        if self.transport is None:
            # XXX This _ought_ to be horribly broken but in fact it is
            # not. TODO: Investigate further.  -glyph
            return False
        if self.transport.disconnecting:
            return True
        return False

    def setRawMode(self):
        self.lineMode = False

    def dataReceived(self, data):
        buffer = self.buffer
        buffer += data
        delimiter = self.delimiter
        begin = 0
        raw = False
        while self.lineMode:
            end = buffer.find(delimiter, begin)
            if end == -1:
                break
            line = buffer[begin:end]
            self.lineReceived(line)
            if self.isDisconnecting():
                self.buffer = ''
                return
            begin = end + len(delimiter)
        else:
            raw = True
        if begin:
            buffer = buffer[begin:]
        if raw:
            self.buffer = ''
            if self.isDisconnecting():
                return
            if buffer:
                self.rawDataReceived(buffer)
        else:
            self.buffer = buffer





# Copyright 2005 Divmod, Inc.  See LICENSE file for details

def stablesort(self, other):
    return cmp(self.__class__, getattr(other, '__class__', type(other)))

class TCPEndpoint:
    def __init__(self, host, port):
        self.host = host
        self.port = port

    def __hash__(self):
        return hash((self.host, self.port)) + 5

    def connect(self, protocolFactory):
        from twisted.internet import reactor
        return reactor.connectTCP(self.host, self.port, protocolFactory)

    def __repr__(self):
        return '<TCP@%s,%d>' % (self.host, self.port)

    def __cmp__(self, other):
        if isinstance(other, TCPEndpoint):
            return cmp((self.host, self.port),
                       (other.host, other.port))
        return stablesort(self, other)


class Q2QEndpoint:
    def __init__(self, service, fromAddress, toAddress, protocolName):
        self.service = service
        self.fromAddress = fromAddress
        self.toAddress = toAddress
        self.protocolName = protocolName

    def __repr__(self):
        return '<Q2Q from <%s> to <%s> on %r>' % (
            self.fromAddress, self.toAddress, self.protocolName)

    def __cmp__(self, other):
        if isinstance(other, Q2QEndpoint):
            return cmp((self.fromAddress, self.toAddress, self.protocolName),
                       (other.fromAddress, other.toAddress, other.protocolName))
        return stablesort(self, other)

    def __hash__(self):
        return hash((self.fromAddress,
                     self.toAddress,
                     self.protocolName)) + 7

    def connect(self, protocolFactory):
        # from twisted.python.context import get
        # get("q2q-service")
        return self.service.connectQ2Q(
            self.fromAddress, self.toAddress, self.protocolName,
            protocolFactory)




# -*- test-case-name: vertex.test.test_ptcp -*-

import struct, zlib
import itertools

from epsilon.pending import PendingEvent

from twisted.internet import protocol, error, reactor, defer
from twisted.internet.main import CONNECTION_DONE
from twisted.python import log, util

from vertex import tcpdfa
from vertex.statemachine import StateError


genConnID = itertools.count(8).next

MAX_PSEUDO_PORT = (2 ** 16)

_packetFormat = ('!' # WTF did you think
                 'H' # sourcePseudoPort
                 'H' # destPseudoPort
                 'L' # sequenceNumber
                 'L' # acknowledgementNumber
                 'L' # window
                 'B' # flags
                 'l' # checksum
                     # (signed because of crc32 module)
                 'H' # dlen
                 )
_fixedSize = struct.calcsize(_packetFormat)

_SYN, _ACK, _FIN, _RST, _STB = [1 << n for n in range(5)]

def _flagprop(flag):
    def setter(self, value):
        if value:
            self.flags |= flag
        else:
            self.flags &= ~flag
    return property(lambda self: bool(self.flags & flag), setter)

def relativeSequence(wireSequence, initialSequence, lapNumber):
    """ Compute a relative sequence number from a wire sequence number so that we
    can use natural Python comparisons on it, such as <, >, ==.

    @param wireSequence: the sequence number received on the wire.

    @param initialSequence: the ISN for this sequence, negotiated at SYN time.

    @param lapNumber: the number of times that this value has wrapped around
    2**32.
    """
    return (wireSequence + (lapNumber * (2**32))) - initialSequence

class PTCPPacket(util.FancyStrMixin, object):
    showAttributes = (
        ('sourcePseudoPort', 'sourcePseudoPort', '%d'),
        ('destPseudoPort', 'destPseudoPort', '%d'),
        ('shortdata', 'data', '%r'),
        ('niceflags', 'flags', '%s'),
        ('dlen', 'dlen', '%d'),
        ('seqNum', 'seq', '%d'),
        ('ackNum', 'ack', '%d'),
        ('checksum', 'checksum', '%x'),
        ('peerAddressTuple', 'peerAddress', '%r'),
        ('retransmitCount', 'retransmitCount', '%d'),
        )

    syn = _flagprop(_SYN)
    ack = _flagprop(_ACK)
    fin = _flagprop(_FIN)
    rst = _flagprop(_RST)
    stb = _flagprop(_STB)

    # Number of retransmit attempts left for this segment.  When it reaches
    # zero, this segment is dead.
    retransmitCount = 50

    def shortdata():
        def get(self):
            if len(self.data) > 13:
                return self.data[:5] + '...' + self.data[-5:]
            else:
                return self.data
        return get,
    shortdata = property(*shortdata())

    def niceflags():
        def get(self):
            res = []
            for (f, v) in [
                (self.syn, 'S'), (self.ack, 'A'), (self.fin, 'F'),
                (self.rst, 'R'), (self.stb, 'T')]:
                res.append(f and v or '.')
            return ''.join(res)
        return get,
    niceflags = property(*niceflags())

    def create(cls,
               sourcePseudoPort, destPseudoPort,
               seqNum, ackNum, data,
               window=(1 << 15),
               syn=False, ack=False, fin=False,
               rst=False, stb=False,
               destination=None):
        i = cls(sourcePseudoPort, destPseudoPort,
                seqNum, ackNum, window,
                0, 0, len(data), data)
        i.syn = syn
        i.ack = ack
        i.fin = fin
        i.rst = rst
        i.stb = stb
        i.checksum = i.computeChecksum()
        i.destination = destination
        return i
    create = classmethod(create)


    def __init__(self,
                 sourcePseudoPort,
                 destPseudoPort,
                 seqNum, ackNum, window, flags,
                 checksum, dlen, data, peerAddressTuple=None,
                 seqOffset=0, ackOffset=0, seqLaps=0, ackLaps=0):
        self.sourcePseudoPort = sourcePseudoPort
        self.destPseudoPort = destPseudoPort
        self.seqNum = seqNum
        self.ackNum = ackNum
        self.window = window
        self.flags = flags
        self.checksum = checksum
        self.dlen = dlen
        self.data = data
        self.peerAddressTuple = peerAddressTuple # None if local

        self.seqOffset = seqOffset
        self.ackOffset = ackOffset
        self.seqLaps = seqLaps
        self.ackLaps = ackLaps

    def segmentLength(self):
        """RFC page 26: 'The segment length (SEG.LEN) includes both data and sequence
        space occupying controls'
        """
        return self.dlen + self.syn + self.fin

    def relativeSeq(self):
        return relativeSequence(self.seqNum, self.seqOffset, self.seqLaps)

    def relativeAck(self):
        return relativeSequence(self.ackNum, self.ackOffset, self.ackLaps)


    def verifyChecksum(self):
        return len(self.data) == self.dlen and self.checksum == self.computeChecksum()


    def computeChecksum(self):
        return zlib.crc32(self.data)

    def decode(cls, bytes, hostPortPair):
        fields = struct.unpack(_packetFormat, bytes[:_fixedSize])
        sourcePseudoPort, destPseudoPort, seq, ack, window, flags, checksum, dlen = fields
        data = bytes[_fixedSize:]
        return cls(sourcePseudoPort, destPseudoPort, seq, ack, window, flags,
                   checksum, dlen, data, hostPortPair)
    decode = classmethod(decode)

    def mustRetransmit(self):
        """Check to see if this packet must be retransmitted until it was received.
        """
        if self.syn or self.fin or self.dlen:
            return True
        return False

    def encode(self):
        dlen = len(self.data)
        checksum = self.computeChecksum()
        return struct.pack(
            _packetFormat,
            self.sourcePseudoPort, self.destPseudoPort,
            self.seqNum, self.ackNum, self.window,
            self.flags, checksum, dlen) + self.data

    def fragment(self, mtu):
        if self.dlen < mtu:
            return [self]
        assert not self.syn, "should not be originating syn packets w/ data"
        seqOfft = 0
        L = []
        # XXX TODO: need to take seqLaps into account, etc.
        for chunk in iterchunks(self.data, mtu):
            last = self.create(self.sourcePseudoPort,
                               self.destPseudoPort,
                               self.seqNum + seqOfft,
                               self.ackNum,
                               chunk,
                               self.window,
                               destination=self.destination,
                               ack=self.ack)
            L.append(last)
            seqOfft += len(chunk)
        if self.fin:
            last.fin = self.fin
            last.checksum = last.computeChecksum()
        return L


def iterchunks(data, chunksize):
    """iterate chunks of data
    """
    offt = 0
    while offt < len(data):
        yield data[offt:offt+chunksize]
        offt += chunksize


def ISN():
    """
    Initial Sequence Number generator.
    """
    # return int((time.time() * 1000000) / 4) % 2**32
    return 0


def segmentAcceptable(RCV_NXT, RCV_WND, SEG_SEQ, SEG_LEN):
    # RFC page 26.
    if SEG_LEN == 0 and RCV_WND == 0:
        return SEG_SEQ == RCV_NXT
    if SEG_LEN == 0 and RCV_WND > 0:
        return ((RCV_NXT <= SEG_SEQ) and (SEG_SEQ < RCV_NXT + RCV_WND))
    if SEG_LEN > 0 and RCV_WND == 0:
        return False
    if SEG_LEN > 0 and RCV_WND > 0:
        return ((  (RCV_NXT <= SEG_SEQ) and (SEG_SEQ < RCV_NXT + RCV_WND))
                or ((RCV_NXT <= SEG_SEQ+SEG_LEN-1) and
                    (SEG_SEQ+SEG_LEN-1 < RCV_NXT + RCV_WND)))
    assert 0, 'Should be impossible to get here.'
    return False

class BadPacketError(Exception):
    """
    """

class PTCPConnection(tcpdfa.TCP):
    """
    Implementation of RFC 793 state machine.

    @ivar oldestUnackedSendSeqNum: (TCP RFC: SND.UNA) The oldest (relative)
    sequence number referring to an octet which we have sent or may send which
    is unacknowledged.  This begins at 0, which is special because it is not
    for an octet, but rather for the initial SYN packet.  Unless it is 0, this
    represents the sequence number of self._outgoingBytes[0].

    @ivar nextSendSeqNum: (TCP RFC: SND.NXT) The next (relative) sequence
    number that we will send to our peer after the current buffered segments
    have all been acknowledged.  This is the sequence number of the
    not-yet-extant octet in the stream at
    self._outgoingBytes[len(self._outgoingBytes)].

    @ivar nextRecvSeqNum: (TCP RFC: RCV.NXT) The next (relative) sequence
    number that the peer should send to us if they want to send more data;
    their first unacknowledged sequence number as far as we are concerned; the
    left or lower edge of the receive window; the sequence number of the first
    octet that has not been delivered to the application.  changed whenever we
    receive an appropriate ACK.

    @ivar peerSendISN: the initial sequence number that the peer sent us during
    the negotiation phase.  All peer-relative sequence numbers are computed
    using this.  (see C{relativeSequence}).

    @ivar hostSendISN: the initial sequence number that the we sent during the
    negotiation phase.  All host-relative sequence numbers are computed using
    this.  (see C{relativeSequence})

    @ivar retransmissionQueue: a list of packets to be re-sent until their
    acknowledgements come through.

    @ivar recvWindow: (TCP RFC: RCV.WND) - the size [in octets] of the current
    window allowed by this host, to be in transit from the other host.

    @ivar sendWindow: (TCP RFC: SND.WND) - the size [in octets] of the current
    window allowed by our peer, to be in transit from us.

    """

    mtu = 512 - _fixedSize

    recvWindow = mtu
    sendWindow = mtu
    sendWindowRemaining = mtu * 2

    protocol = None

    def __init__(self,
                 hostPseudoPort, peerPseudoPort,
                 ptcp, factory, peerAddressTuple):
        tcpdfa.TCP.__init__(self)
        self.hostPseudoPort = hostPseudoPort
        self.peerPseudoPort = peerPseudoPort
        self.ptcp = ptcp
        self.factory = factory
        self._receiveBuffer = []
        self.retransmissionQueue = []
        self.peerAddressTuple = peerAddressTuple

        self.oldestUnackedSendSeqNum = 0
        self.nextSendSeqNum = 0
        self.hostSendISN = 0
        self.nextRecvSeqNum = 0
        self.peerSendISN = 0
        self.setPeerISN = False

    peerSendISN = None

    def packetReceived(self, packet):
        # XXX TODO: probably have to do something to the packet here to
        # identify its relative sequence number.

        # print 'received', self, packet

        if packet.stb:
            # Shrink the MTU
            [self.mtu] = struct.unpack('!H', packet.data)
            rq = []
            for pkt in self.retransmissionQueue:
                rq.extend(pkt.fragment(self.mtu))
            self.retransmissionQueue = rq
            return

        if self._paused:
            return

        generatedStateMachineInput = False
        if packet.syn:
            if packet.dlen:
                # Whoops, what?  SYNs probably can contain data, I think, but I
                # certainly don't see anything in the spec about how to deal
                # with this or in ethereal for how linux deals with it -glyph
                raise BadPacketError(
                    "currently no data allowed in SYN packets: %r"
                    % (packet,))
            else:
                assert packet.segmentLength() == 1
            if self.peerAddressTuple is None:
                # we're a server
                assert self.wasEverListen, "Clients must specify a connect address."
                self.peerAddressTuple = packet.peerAddressTuple
            else:
                # we're a client
                assert self.peerAddressTuple == packet.peerAddressTuple
            if self.setPeerISN:
                if self.peerSendISN != packet.seqNum:
                    raise BadPacketError(
                        "Peer ISN was already set to %s but incoming packet "
                        "tried to set it to %s" % (
                            self.peerSendISN, packet.seqNum))
                if not self.retransmissionQueue:
                    # If our retransmissionQueue is hot, we are going to send
                    # them an ACK to this with the next packet we send them
                    # anyway; as a bonus, this will properly determine whether
                    # we're sending a SYN+ACK or merely an ACK; the only time
                    # we send an ACK is when we have nothing to say to them and
                    # they're blocked on getting a response to their SYN+ACK
                    # from us. -glyph
                    self.originate(ack=True)
                return
            self.setPeerISN = True
            self.peerSendISN = packet.seqNum
            # syn, fin, and data are mutually exclusive, so this relative
            # sequence-number increment is done both here, and below in the
            # data/fin processing block.
            self.nextRecvSeqNum += packet.segmentLength()
            if not packet.ack:
                generatedStateMachineInput = True
                self.input(tcpdfa.SYN)

        SEG_ACK = packet.relativeAck() # aliasing this for easier reading w/
                                       # the RFC
        if packet.ack:
            if (self.oldestUnackedSendSeqNum < SEG_ACK and
                SEG_ACK <= self.nextSendSeqNum):
                # According to the spec, an 'acceptable ack
                rq = self.retransmissionQueue
                while rq:
                    segmentOnQueue = rq[0]
                    qSegSeq = segmentOnQueue.relativeSeq()
                    if qSegSeq + segmentOnQueue.segmentLength() <= SEG_ACK:
                        # fully acknowledged, as per RFC!
                        rq.pop(0)
                        sminput = None
                        self.sendWindowRemaining += segmentOnQueue.segmentLength()
                        # print 'inc send window', self, self.sendWindowRemaining
                        if segmentOnQueue.syn:
                            if packet.syn:
                                sminput = tcpdfa.SYN_ACK
                            else:
                                sminput = tcpdfa.ACK
                        elif segmentOnQueue.fin:
                            sminput = tcpdfa.ACK
                        if sminput is not None:
                            # print 'ack input:', segmentOnQueue, packet, sminput
                            generatedStateMachineInput = True
                            self.input(sminput)
                    else:
                        break
                else:
                    # write buffer is empty; alert the application layer.
                    self._writeBufferEmpty()
                self.oldestUnackedSendSeqNum = SEG_ACK

        if packet.syn:
            assert generatedStateMachineInput
            return

        # XXX TODO: examine 'window' field and adjust sendWindowRemaining
        # is it 'occupying a portion of valid receive sequence space'?  I think
        # this means 'packet which might acceptably contain useful data'
        if not packet.segmentLength():
            assert packet.ack, "What the _HELL_ is wrong with this packet:" +str(packet)
            return

        if not segmentAcceptable(self.nextRecvSeqNum,
                                 self.recvWindow,
                                 packet.relativeSeq(),
                                 packet.segmentLength()):
            # We have to transmit an ack here since it's old data.  We probably
            # need to ack in more states than just ESTABLISHED... but which
            # ones?
            if not self.retransmissionQueue:
                self.originate(ack=True)
            return

        # OK!  It's acceptable!  Let's process the various bits of data.
        # Where is the useful data in the packet?
        if packet.relativeSeq() > self.nextRecvSeqNum:
            # XXX: Here's what's going on.  Data can be 'in the window', but
            # still in the future.  For example, if I have a window of length 3
            # and I send segments DATA1(len 1) DATA2(len 1) FIN and you receive
            # them in the order FIN DATA1 DATA2, you don't actually want to
            # process the FIN until you've processed the data.

            # For the moment we are just dropping anything that isn't exactly
            # the next thing we want to process.  This is perfectly valid;
            # these packets might have been dropped, so the other end will have
            # to retransmit them anyway.
            return

        if packet.dlen:
            assert not packet.syn, 'no seriously I _do not_ know how to handle this'
            usefulData = packet.data[self.nextRecvSeqNum - packet.relativeSeq():]
            # DONT check/slice the window size here, the acceptability code
            # checked it, we can over-ack if the other side is buggy (???)
            if self.protocol is not None:
                try:
                    self.protocol.dataReceived(usefulData)
                except:
                    log.err()
                    self.loseConnection()

        self.nextRecvSeqNum += packet.segmentLength()
        if self.state == tcpdfa.ESTABLISHED:
            # In all other states, the state machine takes care of sending ACKs
            # in its output process.
            self.originate(ack=True)

        if packet.fin:
            self.input(tcpdfa.FIN)


    def getHost(self):
        tupl = self.ptcp.transport.getHost()
        return PTCPAddress((tupl.host, tupl.port),
                           self.pseudoPortPair)

    def getPeer(self):
        return PTCPAddress(self.peerAddressTuple,
                           self.pseudoPortPair)

    _outgoingBytes = ''
    _nagle = None

    def write(self, bytes):
        assert not self.disconnected, 'Writing to a transport that was already disconnected.'
        self._outgoingBytes += bytes
        self._writeLater()


    def writeSequence(self, seq):
        self.write(''.join(seq))


    def _writeLater(self):
        if self._nagle is None:
            self._nagle = reactor.callLater(0.001, self._reallyWrite)

    def _originateOneData(self):
        amount = min(self.sendWindowRemaining, self.mtu)
        sendOut = self._outgoingBytes[:amount]
        # print 'originating data packet', len(sendOut)
        self._outgoingBytes = self._outgoingBytes[amount:]
        self.sendWindowRemaining -= len(sendOut)
        self.originate(ack=True, data=sendOut)

    def _reallyWrite(self):
        # print self, 'really writing', self._paused
        self._nagle = None
        if self._outgoingBytes:
            # print 'window and bytes', self.sendWindowRemaining, len(self._outgoingBytes)
            while self.sendWindowRemaining and self._outgoingBytes:
                self._originateOneData()

    _retransmitter = None
    _retransmitTimeout = 0.5

    def _retransmitLater(self):
        assert self.state != tcpdfa.CLOSED
        if self._retransmitter is None:
            self._retransmitter = reactor.callLater(self._retransmitTimeout, self._reallyRetransmit)

    def _stopRetransmitting(self):
        # used both as a quick-and-dirty test shutdown hack and a way to shut
        # down when we die...
        if self._retransmitter is not None:
            self._retransmitter.cancel()
            self._retransmitter = None
        if self._nagle is not None:
            self._nagle.cancel()
            self._nagle = None
        if self._closeWaitLoseConnection is not None:
            self._closeWaitLoseConnection.cancel()
            self._closeWaitLoseConnection = None

    def _reallyRetransmit(self):
        # XXX TODO: packet fragmentation & coalescing.
        # print 'Wee a retransmit!  What I got?', self.retransmissionQueue
        self._retransmitter = None
        if self.retransmissionQueue:
            for packet in self.retransmissionQueue:
                packet.retransmitCount -= 1
                if packet.retransmitCount:
                    packet.ackNum = self.currentAckNum()
                    self.ptcp.sendPacket(packet)
                else:
                    self.input(tcpdfa.TIMEOUT)
                    return
            self._retransmitLater()

    disconnecting = False       # This is *TWISTED* level state-machine stuff,
                                # not TCP-level.

    def loseConnection(self):
        if not self.disconnecting:
            self.disconnecting = True
            if not self._outgoingBytes:
                self._writeBufferEmpty()


    def _writeBufferEmpty(self):
        if self._outgoingBytes:
            self._reallyWrite()
        elif self.producer is not None:
            if (not self.streamingProducer) or self.producerPaused:
                self.producerPaused = False
                self.producer.resumeProducing()
        elif self.disconnecting and (not self.disconnected
                                     or self.state == tcpdfa.CLOSE_WAIT):
            self.input(tcpdfa.APP_CLOSE)


    def _writeBufferFull(self):
        # print 'my write buffer is full'
        if (self.producer is not None
            and not self.producerPaused):
            self.producerPaused = True
            # print 'producer pausing'
            self.producer.pauseProducing()
            # print 'producer paused'
        else:
            # print 'but I am not telling my producer to pause!'
            # print '  ', self.producer, self.streamingProducer, self.producerPaused
            pass


    disconnected = False
    producer = None
    producerPaused = False
    streamingProducer = False

    def registerProducer(self, producer, streaming):
        if self.producer is not None:
            raise RuntimeError(
                "Cannot register producer %s, "
                "because producer %s was never unregistered."
                % (producer, self.producer))
        if self.disconnected:
            producer.stopProducing()
        else:
            self.producer = producer
            self.streamingProducer = streaming
            if not streaming and not self._outgoingBytes:
                producer.resumeProducing()

    def unregisterProducer(self):
        self.producer = None
        if not self._outgoingBytes:
            self._writeBufferEmpty()

    _paused = False
    def pauseProducing(self):
        self._paused = True

    def resumeProducing(self):
        self._paused = False

    def currentAckNum(self):
        return (self.nextRecvSeqNum + self.peerSendISN) % (2**32)

    def originate(self, data='', syn=False, ack=False, fin=False):
        if syn:
            # We really should be randomizing the ISN but until we finish the
            # implementations of the various bits of wraparound logic that were
            # started with relativeSequence
            assert self.nextSendSeqNum == 0
            assert self.hostSendISN == 0
        p = PTCPPacket.create(self.hostPseudoPort,
                              self.peerPseudoPort,
                              seqNum=(self.nextSendSeqNum + self.hostSendISN) % (2**32),
                              ackNum=self.currentAckNum(),
                              data=data,
                              window=self.recvWindow,
                              syn=syn, ack=ack, fin=fin,
                              destination=self.peerAddressTuple)
        # do we want to enqueue this packet for retransmission?
        sl = p.segmentLength()
        self.nextSendSeqNum += sl

        if p.mustRetransmit():
            # print self, 'originating retransmittable packet', len(self.retransmissionQueue)
            if self.retransmissionQueue:
                if self.retransmissionQueue[-1].fin:
                    raise AssertionError("Sending %r after FIN??!" % (p,))
            # print 'putting it on the queue'
            self.retransmissionQueue.append(p)
            # print 'and sending it later'
            self._retransmitLater()
            if not self.sendWindowRemaining: # len(self.retransmissionQueue) > 5:
                # print 'oh no my queue is too big'
                # This is a random number (5) because I ought to be summing the
                # packet lengths or something.
                self._writeBufferFull()
            else:
                # print 'my queue is still small enough', len(self.retransmissionQueue), self, self.sendWindowRemaining
                pass
        self.ptcp.sendPacket(p)

    # State machine transition definitions, hooray.
    def transition_SYN_SENT_to_CLOSED(self):
        """
        The connection never got anywhere.  Goodbye.
        """
        # XXX CONNECTOR API OMFG
        self.factory.clientConnectionFailed(None, error.TimeoutError())


    wasEverListen = False

    def enter_LISTEN(self):
        # Spec says this is necessary for RST handling; we need it for making
        # sure it's OK to bind port numbers.
        self.wasEverListen = True

    def enter_CLOSED(self):
        self.ptcp.connectionClosed(self)
        self._stopRetransmitting()
        if self._timeWaitCall is not None:
            self._timeWaitCall.cancel()
            self._timeWaitCall = None

    _timeWaitCall = None
    _timeWaitTimeout = 0.01     # REALLY fast timeout, right now this is for
                                # the tests...

    def enter_TIME_WAIT(self):
        self._stopRetransmitting()
        self._timeWaitCall = reactor.callLater(self._timeWaitTimeout, self._do2mslTimeout)

    def _do2mslTimeout(self):
        self._timeWaitCall = None
        self.input(tcpdfa.TIMEOUT)

    peerAddressTuple = None

    def transition_LISTEN_to_SYN_SENT(self):
        """
        Uh, what?  We were listening and we tried to send some bytes.
        This is an error for PTCP.
        """
        raise StateError("You can't write anything until someone connects to you.")

#     def invalidInput(self, datum):
#         print self, self.protocol, 'invalid input', datum

    def pseudoPortPair():
        def get(self):
            return (self.hostPseudoPort,
                    self.peerPseudoPort)
        return get,
    pseudoPortPair = property(*pseudoPortPair())

    def enter_ESTABLISHED(self):
        """
        We sent out SYN, they acknowledged it.  Congratulations, you
        have a new baby connection.
        """
        assert not self.disconnecting
        assert not self.disconnected
        try:
            p = self.factory.buildProtocol(PTCPAddress(
                    self.peerAddressTuple, self.pseudoPortPair))
            p.makeConnection(self)
        except:
            log.msg("Exception during PTCP connection setup.")
            log.err()
            self.loseConnection()
        else:
            self.protocol = p

    def exit_ESTABLISHED(self):
        assert not self.disconnected
        self.disconnected = True
        try:
            self.protocol.connectionLost(CONNECTION_DONE)
        except:
            log.err()
        self.protocol = None

        if self.producer is not None:
            try:
                self.producer.stopProducing()
            except:
                log.err()
            self.producer = None


    _closeWaitLoseConnection = None

    def enter_CLOSE_WAIT(self):
        # Twisted automatically reacts to network half-close by issuing a full
        # close.
        self._closeWaitLoseConnection = reactor.callLater(0.01, self._loseConnectionBecauseOfCloseWait)

    def _loseConnectionBecauseOfCloseWait(self):
        self._closeWaitLoseConnection = None
        self.loseConnection()

    def immediateShutdown(self):
        """_IMMEDIATELY_ shut down this connection, sending one (non-retransmitted)
        app-close packet, emptying our buffers, clearing our producer and
        getting ready to die right after this call.
        """
        self._outgoingBytes = ''
        if self.state == tcpdfa.ESTABLISHED:
            self.input(tcpdfa.APP_CLOSE)
            self._stopRetransmitting()
            self._reallyRetransmit()

        # All states that we can reasonably be in handle a timeout; force our
        # connection to think that it's become desynchronized with the other
        # end so that it will totally shut itself down.

        self.input(tcpdfa.TIMEOUT)
        assert self._retransmitter is None
        assert self._nagle is None

    def output_ACK(self):
        self.originate(ack=True)

    def output_FIN(self):
        self.originate(fin=True)

    def output_SYN_ACK(self):
        self.originate(syn=True, ack=True)

    def output_SYN(self):
        self.originate(syn=True)

class PTCPAddress(object):
    # garbage

    def __init__(self, (host, port), (pseudoHostPort, pseudoPeerPort)):
        self.host = host
        self.port = port
        self.pseudoHostPort = pseudoHostPort
        self.pseudoPeerPort = pseudoPeerPort

    def __repr__(self):
        return 'PTCPAddress((%r, %r), (%r, %r))' % (
            self.host, self.port,
            self.pseudoHostPort,
            self.pseudoPeerPort)

class PTCP(protocol.DatagramProtocol):
    # External API

    def __init__(self, factory):
        self.factory = factory
        self._allConnectionsClosed = PendingEvent()

    def connect(self, factory, host, port, pseudoPort=1):
        sourcePseudoPort = genConnID() % MAX_PSEUDO_PORT
        conn = self._connections[(pseudoPort, sourcePseudoPort, (host, port))
                                 ] = PTCPConnection(
            sourcePseudoPort, pseudoPort, self, factory, (host, port))
        conn.input(tcpdfa.APP_ACTIVE_OPEN)
        return conn

    def sendPacket(self, packet):
        if self.transportGoneAway:
            return
        self.transport.write(packet.encode(), packet.destination)


    # Internal stuff
    def startProtocol(self):
        self.transportGoneAway = False
        self._lastConnID = 10 # random.randrange(2 ** 32)
        self._connections = {}

    def _finalCleanup(self):
        """
        Clean up all of our connections by issuing application-level close and
        stop notifications, sending hail-mary final FIN packets (which may not
        reach the other end, but nevertheless can be useful) when possible.
        """
        for conn in self._connections.values():
            conn.immediateShutdown()
        assert not self._connections

    def stopProtocol(self):
        """
        Notification from twisted that our underlying port has gone away;
        make sure we're not going to try to send any packets through our
        transport and blow up, then shut down all of our protocols, issuing
        appr
        opriate application-level messages.
        """
        self.transportGoneAway = True
        self._finalCleanup()

    def cleanupAndClose(self):
        """
        Clean up all remaining connections, then close our transport.

        Although in a pinch we will do cleanup after our socket has gone away
        (if it does so unexpectedly, above in stopProtocol), we would really
        prefer to do cleanup while we still have access to a transport, since
        that way we can force out a few final packets and save the remote
        application an awkward timeout (if it happens to get through, which
        is generally likely).
        """
        self._finalCleanup()
        return self._stop()

    def datagramReceived(self, bytes, addr):
        if len(bytes) < _fixedSize:
            # It can't be any good.
            return

        pkt = PTCPPacket.decode(bytes, addr)

        if pkt.dlen > len(pkt.data):
            self.sendPacket(
                PTCPPacket.create(
                    pkt.destPseudoPort,
                    pkt.sourcePseudoPort,
                    0,
                    0,
                    struct.pack('!H', len(pkt.data)),
                    stb=True,
                    destination=addr))
        elif not pkt.verifyChecksum():
            print "bad packet", pkt
            print pkt.dlen, len(pkt.data)
            print repr(pkt.data)
            print hex(pkt.checksum), hex(pkt.computeChecksum())
        else:
            self.packetReceived(pkt)

    stopped = False
    def _stop(self, result=None):
        if not self.stopped:
            self.stopped = True
            return self.transport.stopListening()
        else:
            return defer.succeed(None)

    def waitForAllConnectionsToClose(self):
        """
        Wait for all currently-open connections to enter the 'CLOSED' state.
        Currently this is only usable from test fixtures.
        """
        if not self._connections:
            return self._stop()
        return self._allConnectionsClosed.deferred().addBoth(self._stop)

    def connectionClosed(self, ptcpConn):
        packey = (ptcpConn.peerPseudoPort, ptcpConn.hostPseudoPort,
                  ptcpConn.peerAddressTuple)
        del self._connections[packey]
        if ((not self.transportGoneAway) and
            (not self._connections) and
            self.factory is None):
            self._stop()
        if not self._connections:
            self._allConnectionsClosed.callback(None)

    def packetReceived(self, packet):
        packey = (packet.sourcePseudoPort, packet.destPseudoPort, packet.peerAddressTuple)
        if packey not in self._connections:
            if packet.flags == _SYN and packet.destPseudoPort == 1: # SYN and _ONLY_ SYN set.
                conn = PTCPConnection(packet.destPseudoPort,
                                      packet.sourcePseudoPort, self,
                                      self.factory, packet.peerAddressTuple)
                conn.input(tcpdfa.APP_PASSIVE_OPEN)
                self._connections[packey] = conn
            else:
                log.msg("corrupted packet? %r %r %r" % (packet,packey, self._connections))
                return
        try:
            self._connections[packey].packetReceived(packet)
        except:
            log.msg("PTCPConnection error on %r:" % (packet,))
            log.err()
            del self._connections[packey]



# Copyright 2005 Divmod, Inc.  See LICENSE file for details

from vertex import juice

class NotAllowed(Exception):
    pass

class AddUser(juice.Command):
    """
    Add a user to a domain.
    """
    commandName = "add_user"

    arguments = [
        ("name", juice.String()),
        ("password", juice.String())
        ]

    response = []

    errors = {NotAllowed: "NotAllowed"}





from twisted.internet.defer import Deferred
try:
    from twisted.internet.defer import FirstError
except:
    class FirstError(Exception):
        """omgwtf
        """
from twisted.python import failure

class NoFailure(Exception):
    """ This is a placeholder exception used by DeferredList to indicate that there
    is not actually a failure.
    """

class DeferredList(Deferred):
    """I combine a group of deferreds into one callback.

    I track a list of L{Deferred}s for their callbacks, and make a single
    callback when they have all completed, a list of (success, result)
    tuples, 'success' being a boolean.

    Note that you can still use a L{Deferred} after putting it in a
    DeferredList.  For example, you can suppress 'Unhandled error in Deferred'
    messages by adding errbacks to the Deferreds *after* putting them in the
    DeferredList, as a DeferredList won't swallow the errors.  (Although a more
    convenient way to do this is simply to set the consumeErrors flag)
    """

    fireOnOneCallback = 0
    fireOnOneErrback = 0

    def __init__(self, deferredList, fireOnOneCallback=0, fireOnOneErrback=0,
                 consumeErrors=0):
        """Initialize a DeferredList.

        @type deferredList:  C{list} of L{Deferred}s
        @param deferredList: The list of deferreds to track.
        @param fireOnOneCallback: (keyword param) a flag indicating that
                             only one callback needs to be fired for me to call
                             my callback
        @param fireOnOneErrback: (keyword param) a flag indicating that
                            only one errback needs to be fired for me to call
                            my errback
        @param consumeErrors: (keyword param) a flag indicating that any errors
                            raised in the original deferreds should be
                            consumed by this DeferredList.  This is useful to
                            prevent spurious warnings being logged.
        """
        self.resultList = [None] * len(deferredList)
        Deferred.__init__(self)
        if len(deferredList) == 0 and not fireOnOneCallback:
            self.callback(self.resultList)

        # These flags need to be set *before* attaching callbacks to the
        # deferreds, because the callbacks use these flags, and will run
        # synchronously if any of the deferreds are already fired.
        self.fireOnOneCallback = fireOnOneCallback
        self.fireOnOneErrback = fireOnOneErrback
        self.consumeErrors = consumeErrors
        self.finishedCount = 0

        index = 0
        for deferred in deferredList:
            deferred.addCallbacks(self._cbDeferred, self._cbDeferred,
                                  callbackArgs=(index,SUCCESS),
                                  errbackArgs=(index,FAILURE))
            index = index + 1

    def _cbDeferred(self, result, index, succeeded):
        """(internal) Callback for when one of my deferreds fires.
        """
        self.resultList[index] = (succeeded, result)

        self.finishedCount += 1
        if not self.called:
            if succeeded == SUCCESS and self.fireOnOneCallback:
                # build a convincing list:
                for rli in xrange(len(self.resultList)):
                    if self.resultList[rli] is None:
                        self.resultList[rli] = (0, failure.Failure(NoFailure()))
                self.callback(self.resultList)
            elif succeeded == FAILURE and self.fireOnOneErrback:
                self.errback(failure.Failure(FirstError(result, index)))
            elif self.finishedCount == len(self.resultList):
                self.callback(self.resultList)

        if succeeded == FAILURE and self.consumeErrors:
            result = None

        return result

# Constants for use with DeferredList

SUCCESS = True
FAILURE = False







# Copyright 2005 Divmod, Inc.  See LICENSE file for details

import time
import os
import sys
import struct
import getpass

from vertex import q2q, juice, sigma
from twisted.python.usage import Options

from twisted.python import log
from twisted.internet import reactor
from twisted.internet import protocol
from twisted.internet.task import LoopingCall
from twisted.internet import error
from vertex.q2qadmin import AddUser

class Q2QAuthorize(Options):
    def parseArgs(self, who, password=None):
        self.who = who
        self.password = password

    def reportNoCertificate(self, error):
        print "No certificate retrieved:", error.getErrorMessage(), "(see ~/.q2q-client-log for details)"
        log.err(error)
        return None

    def postOptions(self):
        def go():
            self.parent.getService().authorize(
                q2q.Q2QAddress.fromString(self.who),
                self.password).addErrback(self.reportNoCertificate).addCallback(lambda x: reactor.stop())

        if self.password is None:
            self.password = getpass.getpass()

        reactor.callWhenRunning(go)
        self.parent.start()


class BandwidthEstimator:
    bufsize = 20
    totalBytes = 0
    def __init__(self, message, length):
        self.length = length
        self.message = message
        self.estim = []
        self.bytes = 0
        self.call = LoopingCall(self.estimateBandwidth)
        self.call.start(1)

    def estimateBandwidth(self):
        bytes = self.bytes
        self.totalBytes += bytes
        self.estim.append(bytes)
        self.message("%0.2f k/s (%0.2d%%)"
                     % ((sum(self.estim) / len(self.estim)) / 1024.,
                        (float(self.totalBytes) / self.length) * 100))
        if len(self.estim) > self.bufsize:
            self.estim.pop(0)
        self.bytes = 0

    def stop(self):
        self.call.stop()
        self.estimateBandwidth()
        self.message("Finished receiving: %d bytes (%d%%)" % (
                self.totalBytes, (float(self.totalBytes) / self.length) * 100))

class FileReceiver(protocol.Protocol):
    gotLength = False
    estimator = None

    def connectionMade(self):
        self.f = open(self.factory.program.filename, 'wb')
        self.factory.program.parent.info("Started receiving...")

    def dataReceived(self, data):
        if not self.gotLength:
            self.length ,= struct.unpack("!Q", data[:8])
            data = data[8:]
            self.estimator = BandwidthEstimator(self.factory.program.parent.info,
                                                self.length)
            self.gotLength = True

        self.estimator.bytes += len(data)
        self.f.write(data)

    def connectionLost(self, reason):
        self.f.close()
        if self.estimator:
            self.estimator.stop()
        reactor.stop()

from twisted.protocols.basic import FileSender as fsdr

class FileSender(protocol.Protocol):
    def connectionMade(self):
        self.file = self.factory.openFile()
        self.file.seek(0, 2)
        self.length = self.file.tell()
        self.file.seek(0)
        self.estimator = BandwidthEstimator(self.factory.program.parent.info,
                                            self.length)
        self.transport.write(struct.pack("!Q", self.length))
        fsdr().beginFileTransfer(
            self.file, self).addCallback(
            lambda x: self.done())

    def done(self):
        self.factory.program.parent.info("Done sending data: %d bytes" % (
                self.file.tell(),))
        self.transport.loseConnection()

    def dataReceived(self, data):
        print "WTF THE CLIENT IS GETTING DATA", repr(data)

    def registerProducer(self, producer, streaming):
        self.transport.registerProducer(producer, streaming)

    def unregisterProducer(self):
        self.transport.unregisterProducer()

    def write(self, data):
        self.estimator.bytes += len(data)
        self.transport.write(data)

    def connectionLost(self, reason):
        reactor.stop()

class FileSenderFactory(protocol.ClientFactory):
    protocol = FileSender

    def __init__(self, sendprogram):
        self.program = sendprogram

    def openFile(self):
        return file(self.program.filename, 'r')

    def clientConnectionFailed(self, connector, reason):
        self.program.parent.info(
            "Could not connect: %r" % (reason.getErrorMessage(),))
        reactor.stop()

    def clientConnectionLost(self, connector, reason):
        reason.trap(error.ConnectionDone)

class FileReceiverFactory(protocol.Factory):
    def __init__(self, program):
        self.program = program
    protocol = FileReceiver


class ClientCertificateStore(q2q.DirectoryCertificateStore):
    def __init__(self, filepath):
        q2q.DirectoryCertificateStore.__init__(self, os.path.expanduser(filepath))


class ClientQ2QService(q2q.Q2QService):
    def __init__(self, certspath, *a, **kw):
        q2q.Q2QService.__init__(self,
                                certificateStorage=ClientCertificateStore(certspath),
                                q2qPortnum=0,
                                *a, **kw)

    def getDefaultFrom(self, default=None):
        i = self.certificateStorage.localStore.iterkeys()
        try:
            return i.next()
        except StopIteration:
            return default


class TunnelProtocol(protocol.Protocol):
    def __init__(self, tunnel):
        self.tunnel = tunnel
        self.buffer = []

    def connectionMade(self):
        if self.tunnel is not None:
            self.tunnel.setTunnel(self)

    def dataReceived(self, data):
        if self.tunnel is not None:
            self.tunnel.transport.write(data)
        else:
            self.buffer.append(data)

    def setTunnel(self, tunnel):
        if self.tunnel is None:
            self.tunnel = tunnel
            self.dataReceived(''.join(self.buffer))
            del self.buffer
            self.tunnel.setTunnel(self)

class TunnelFactory(protocol.ClientFactory):
    def __init__(self, tunnel):
        self.tunnel = tunnel

    def buildProtocol(self, addr):
        return TunnelProtocol(self.tunnel)

    def clientConnectionFailed(self, connector, reason):
        self.tunnel.transport.loseConnection()
        reactor.stop()

    clientConnectionLost = clientConnectionFailed

class Q2QTunnel(Options):
    optParameters = [
        ['port', 'p', '13000', 'Port on which to start the TCP server'],
        ['destination', 'd', None, 'Q2Q address to which to create the tunnel'],
        ['protocol', 'r', None, 'Q2Q protocol which will operate over the tunnel']]

    def postOptions(self):
        self.toAddr = q2q.Q2QAddress.fromString(self['destination'])

        reactor.listenTCP(int(self['port']), self, interface='127.0.0.1')
        self.parent.start()

    def doStart(self):
        pass

    def doStop(self):
        pass

    def buildProtocol(self, addr):
        p = TunnelProtocol(None)
        svc = self.parent.getService()
        svc.connectQ2Q(self.parent.getFrom(), self.toAddr,
                       self['protocol'], TunnelFactory(p))
        return p

class Q2QReceive(Options):
    optParameters = [["port", "p", "41235", "Port to start the listening server on."]]

    def parseArgs(self, filename):
        self.filename = filename

    def postOptions(self):
        serv = self.parent.getService()
        def pr(x):
            return x
        def stopit(err):
            print "Couldn't Register for File Transfer:", err.getErrorMessage()
            log.err(err)
            reactor.stop()
        serv.listenQ2Q(self.parent.getFrom(),
                       {'file-transfer': FileReceiverFactory(self)},
                       "simple file transfer test").addCallback(pr).addErrback(stopit)
        self.parent.start()

class Q2QSend(Options):

    def parseArgs(self, to, filename):
        self.to = to
        self.filename = filename

    def postOptions(self):
        fs = q2q.Q2QAddress.fromString
        toAddress = fs(self.to)
        fromAddress = self.parent.getFrom()

        toDomain = toAddress.domainAddress()
        svc = self.parent.getService()
        svc.connectQ2Q(fromAddress, toAddress, 'file-transfer',
                       FileSenderFactory(self))
        self.parent.start()


class TextNexusUI(sigma.BaseNexusUI):
    def __init__(self):
        sigma.BaseNexusUI.__init__(self)
        self.call = LoopingCall(self.report)
        self.call.start(5)

    def report(self):
        print 'Transloads:', len(self.transloads)
        for transloadui in self.transloads:
            print '---', transloadui.name, '---'
            print transloadui.bits.percent()
            for peer, mask in transloadui.masks.items():
                print peer, mask.percent()
        print 'end report'

class Q2QSigma(Options):

    def __init__(self, *a, **k):
        Options.__init__(self,*a,**k)
        self.pushers = []

    def opt_push(self, filename):
        self.pushers.append([file(filename), filename, []])

    def opt_to(self, q2qid):
        fs = q2q.Q2QAddress.fromString
        addr = fs(q2qid)
        self.pushers[-1][-1].append(addr)

    def postOptions(self):
        nex = sigma.Nexus(self.parent.getService(),
                          self.parent.getFrom(),
                          TextNexusUI())
        # XXX TODO: there has _GOT_ to be a smarter way to handle text UI for
        # this.
        for sharefile, sharename, sharepeers in self.pushers:
            nex.push(sharefile, sharename, sharepeers)
        self.parent.start()

class UserAdder(juice.Juice):
    def __init__(self):
        juice.Juice.__init__(self, False)

    def connectionMade(self):
        self.d = AddUser(name=self.factory.name,
                         password=self.factory.password).do(self)


class UserAdderFactory(protocol.ClientFactory):
    protocol = UserAdder

    def __init__(self, name, password):
        self.name, self.password = name, password


def enregister(svc, newAddress, password):
    """
    Register a new account and return a Deferred that fires if it worked.

    @param svc: a Q2QService

    @param newAddress: a Q2QAddress object

    @param password: a shared secret (str)
    """
    def trapit(x):
        x.trap(error.ConnectionDone)
    return svc.connectQ2Q(q2q.Q2QAddress("",""),
                       q2q.Q2QAddress(newAddress.domain, "accounts"),
                       'identity-admin',
                       UserAdderFactory(newAddress.resource, password)
                       ).addCallback(
            lambda proto: proto.d).addErrback(
            trapit)

class Q2QRegister(Options):
    synopsis = "<new Q2Q address> <password>"
    def parseArgs(self, newaddress, password):
        self.newaddress = newaddress
        self.password = password

    def postOptions(self):
        fs = q2q.Q2QAddress.fromString
        newAddress = fs(self.newaddress)
        svc = self.parent.getService()

        def showit(x):
            print "%s: %s" % (x.value.__class__, x.getErrorMessage())

        enregister(svc, newAddress, self.password).addErrback(
            showit).addBoth(lambda nothing: reactor.stop())
        self.parent.start()


class Q2QClientProgram(Options):
    subCommands = [
        ['authorize', 'a', Q2QAuthorize, 'Authorize a user'],
        ['register', 'r', Q2QRegister, 'Create a new user '],
        ['tunnel', 't', Q2QTunnel, 'Create an SSL tunnel to a given resource'],
        ['receive', 'l', Q2QReceive, 'Receive for a filetransfer connection'],
        ['send', 's', Q2QSend, 'Send'],
        ['sigma', 'g', Q2QSigma, 'Sigma swarming file-transfer']
        ]

    optParameters = [
        ['from', 'f', None, "Who to send as?"],
        ['tcp', 'p', None, 'TCP port number'],
        ['udp', 'u', 0, 'UDP port number'],
        ['certspath', 'c', "~/.q2qcerts",
         "Path to directory full of public/private certificates."],
        ['logfile', 'l', "~/.q2q-client-log",
         "Path to file where logs of client activity will be written."]
        ]

    optFlags = []

    service = None

    def info(self, message):
        sys.stderr.write(">> %s\n" % (message,))

    def getService(self):
        if self.service is None:
            u = self['udp']
            if u is not None:
                u = int(u)
            t = self['tcp']
            if t is not None:
                t = int(t)
            self.service = ClientQ2QService(self['certspath'],
                                            inboundTCPPortnum=t)
        return self.service

    def getDefaultPath(self):
        return os.path.expanduser(os.path.join(self['certspath'], 'default-address'))

    def getFrom(self):
        fr = self['from']
        if not fr:
            defpath = self.getDefaultPath()
            if os.path.exists(defpath):
                fr = file(defpath).read()
            else:
                fr = self.getService().getDefaultFrom()
                if fr is None:
                    self.info("No default address available, exiting.")
                    self.info(
                        " (Try 'q2q register yourself@divmod.net; "
                        "q2q authorize yourself@divmod.net')")
                    sys.exit(19)
                self.info("Selected default address:"  +fr)
                f = file(defpath, 'wb')
                f.write(fr)
                f.close()

        return q2q.Q2QAddress.fromString(fr)

    def start(self, portno=None):
        import sys
        lfname = self['logfile']
        if lfname == '-':
            lf = sys.stdout
        else:
            lf = file(os.path.expanduser(lfname), 'ab+')
        log.startLogging(lf,
                         setStdout=False)
        srv = self.getService()
        from twisted.application.app import startApplication
        startApplication(srv, False)
        reactor.run()

    verbosity = 0

    def verboseLogger(self, messageDict):
        self.info(' '.join([str(x) for x in messageDict.get('message', [])]))

    def opt_verbose(self):
        self.verbosity += 1
        log.addObserver(log.FileLogObserver(sys.stderr).emit)

    opt_v = opt_verbose



# -*- test-case-name: vertex.test.test_statemachine -*-

NOTHING = 'nothing'             # be quiet (no output)

class StateError(Exception):
    """
    """

class StateMachine:

    initialState = None         # a str describing initial state

    states = None     # dict, mapping state to dict of str input: (str output,
                      # str new-state)


    def __init__(self, initialState=None):
        if initialState is None:
            initialState = self.initialState
        self.state = self.initialState

    def transition(self, oldstate, newstate, datum, *a, **kw):
        if oldstate == newstate:
            return
        # print hex(id(self)), 'Going from', oldstate, 'to', newstate, 'because', datum
        exitmeth = getattr(self, 'exit_%s' % (oldstate,), None)
        entermeth = getattr(self, 'enter_%s' % (newstate,), None)
        transmeth = getattr(self, 'transition_%s_to_%s' % (
                oldstate, newstate), None)
        for meth in exitmeth, entermeth, transmeth:
            if meth is not None:
                meth(*a, **kw)
        self.state = newstate

    def input(self, datum, *a, **kw):
        oldstate = self.state
        if datum == NOTHING:
            return
        try:
            output, newstate = self.states[self.state][datum]
        except KeyError:
            self.invalidInput(datum)
        else:
            OLDSTATE = self.state.upper()
            NEWSTATE = newstate.upper()
            DATUM = datum.upper()
            self.transition(OLDSTATE, NEWSTATE, DATUM, *a, **kw)
            self.output(output, *a, **kw)

    def output(self, datum, *a, **kw):
        foo = getattr(self, 'output_' + datum.upper(), None)
        if foo is not None:
            foo(*a, **kw)

    def invalidInput(self, datum):
        raise StateError("Invalid input in %r: %r" % (self.state, datum))



# Copyright 2005 Divmod, Inc.  See LICENSE file for details
# -*- test-case-name: vertex.test.test_bits -*-
""" The purpose of this module is to provide the class BitArray, a compact
overlay onto an array of bytes which is instead bit-addressable.  It also
includes several bitwise operators.

It does not include all array operations yet, most notably those related to
slicing, since it is written primarily for use by the swarming implementation
and swarming only requires fixed-size bit masks.

"""

__metaclass__ = type

import array
import operator
import math

BITS_PER_BYTE = 8

def operate(operation):
    # XXX TODO: optimize this and countbits later
    def __x__(self, other):
        if len(self) < len(other):
            return operation(other, self)
        new = BitArray(size=len(self))
        for offt, (mybit, hisbit) in enumerate(zip(self, other)):
            result = new[offt] = operation(mybit, hisbit)

        for j in range(offt+1, len(self)):
            new[j] = operation(self[j], 0)
        return new
    return __x__


class BitArray:
    """
    A large mutable array of bits.
    """

    def __init__(self, bytes=None, size=None, default=0):
        if bytes is None and size is None:
            size = 0
        if bytes is None:
            bytes = array.array("B")
            bytesize = int(math.ceil(float(size) / BITS_PER_BYTE))
            if default:
                padbyte = 255
            else:
                padbyte = 0
            bytes.fromlist([padbyte] * bytesize)
        self.bytes = bytes
        if size is None:
            size = len(self.bytes) * self.bytes.itemsize * BITS_PER_BYTE
        self.size = size

        # initialize 'on' and 'off' lists to optimize various things
        self.on = []
        self.off = []
        blists = self.blists = self.off, self.on

        for index, bit in enumerate(self):
            blists[bit].append(index)

    def append(self, bit):
        offt = self.size
        self.size += 1
        if (len(self.bytes) * self.bytes.itemsize * BITS_PER_BYTE) < self.size:
            self.bytes.append(0)
        self[offt] = bit

    def any(self, req=1):
        return bool(self.blists[req])

    def percent(self):
        """
        debugging method; returns a string indicating percentage completion
        """
        if not len(self):
            return 'Inf%'
        return '%0.2f%%'% ((float(self.countbits()) / len(self)) * 100,)

    def __getitem__(self, bitcount):
        if bitcount < 0:
            bitcount += self.size
        if bitcount >= self.size:
            raise IndexError("%r >= %r" % (bitcount, self.size))
        div, mod = divmod(bitcount, self.bytes.itemsize * BITS_PER_BYTE)
        byte = self.bytes[div]
        return (byte >> mod) & 1

    def __setitem__(self, bitcount, bit):
        if bitcount < 0:
            bitcount += self.size
        if bitcount >= self.size:
            raise IndexError("bitcount too big")
        div, mod = divmod(bitcount, self.bytes.itemsize * BITS_PER_BYTE)
        if bit:
            self.bytes[div] |= 1 << mod
        else:
            self.bytes[div] &= ~(1 << mod)

        # change updating
        notbitlist = self.blists[not bit]
        try:
            notbitlist.remove(bitcount)
        except ValueError:
            pass
        bitlist = self.blists[bit]
        if bitcount not in bitlist:
            bitlist.append(bitcount)

    def __len__(self):
        return self.size

    def __repr__(self):
        l = []
        l.append('[')
        for b in self:
            if b:
                c = 'X'
            else:
                c = ' '
            l.append(c)
        l.append(']')
        return ''.join(l)

    def countbits(self, on=True):
        return len(self.blists[on])

    def positions(self, bit):
        """
        An iterator of all positions that a bit holds in this BitArray.

        @param bit: 1 or 0
        """
        return self.blists[bit][:]

    __xor__ = operate(operator.xor)
    __and__ = operate(operator.and_)
    __or__ = operate(operator.or_)




# Copyright 2005 Divmod, Inc.  See LICENSE file for details

from formless.annotate import Typed, InputError
from zope.interface import Interface

class IQ2QTransport(Interface):
    """
    I am a byte-stream-oriented transport which has Q2Q identifiers associated
    with the endpoints, and possibly some cryptographic verification of the
    authenticity of those endpoints.
    """

    def getQ2QHost(self):
        """ Returns a Q2QAddress object representing the user on this end of the
        connection.
        """

    def getQ2QPeer(self):
        """ Returns a Q2QAddress object representing the user on the other end of the
        connection.
        """

class IFileTransfer(Interface):

    def getUploadSink(self, path):
        """
        @param path: a PathFragment that the client wishes to upload to.

        @return: a DataSink where we'll save the data to.
        """

    def getDownloadSource(self, path):
        """
        @param path: a PathFragment that the client wishes to download.

        @return: a DataSource to download data from.
        """

    def listChildren(self, path):
        """
        @param path: a PathFragment that the client wishes to get a list of.

        @return: a list of dictionaries mapping::
            {'name': str,
             'size': int,
             'type': vertex.filexfer.MIMEType,
             'modified': epsilon.extime.Time}
        """

class ISessionTokenStorage(Interface):
    def idFromCookie(self, cookie, domain):
        """Look up a user ID from the given cookie in the given domain.
        """

class ICertificateStorage(Interface):
    def getSelfSignedCertificate(self, domainName):
        """
        @return: a Deferred which will fire with the certificate for the given
        domain name.
        """

    def storeSelfSignedCertificate(self, domainName, mainCert):
        """
        @type mainCert: C{str}
        @param mainCert: Serialized, self-signed certificate to associate
        with the given domain.

        @return: a Deferred which will fire when the certificate has been
        stored successfully.
        """

    def getPrivateCertificate(self, domainName):
        """
        @return: a PrivateCertificate instance, e.g. a certificate including a
        private key, for 'domainName'.
        """

    def addPrivateCertificate(self, domainName, existingCertificate=None):
        """
        """

class Address(Typed):
    def coerce(self, val, configurable):
        try:
            username, domain = val.split("@")
        except ValueError:
            raise InputError("Please enter an address of the form name@domain")
        return username, domain

class IOfferUp(Interface):
    """
    Sharing control database storage.
    """

class IPlugin(Interface):
    """
    """

class ITestPlugin(Interface):
    """
    Dummy plug-in interface for unit testing.
    """

class ITestPlugin2(Interface):
    """
    Dummy plug-in interface for unit testing.
    """



# -*- test-case-name: vertex.test.test_subproducer -*-
# Copyright 2005 Divmod, Inc.  See LICENSE file for details

from twisted.python import log

class SuperProducer:
    """I am a mixin which provides support for mixing in several producers to one
    producer.  I act as a consumer for my producers and as a producer for one
    consumer.

    I must be mixed into a protocol, or something else with a 'transport' attribute.
    """

    producersPaused = False

    def __init__(self):
        self.producingTransports = {}

    def pauseProducing(self):
        self.producersPaused = True
        for transport in self.producingTransports.keys():
            try:
                transport.parentPauseProducing()
            except:
                del self.producingTransports[transport]
                log.err()

    def resumeProducing(self):
        producersWerePaused = self.producersPaused
        if producersWerePaused:
            self.producersPaused = False
        for transport in self.producingTransports.keys():
            try:
                transport.parentResumeProducing()
            except:
                del self.producingTransports[transport]
                log.err()

    def stopProducing(self):
        for transport in self.producingTransports.keys():
            try:
                transport.parentStopProducing()
            except:
                log.err()
        self.producingTransports = {}

    def registerProducerFor(self, trans):
        if not self.producersPaused:
            trans.parentResumeProducing()
        wasProducing = bool(self.producingTransports)
        assert trans not in self.producingTransports
        self.producingTransports[trans] = 1
        if not wasProducing:
            self.transport.registerProducer(self, False)

    def unregisterProducerFor(self, trans):
        if trans in self.producingTransports:
            del self.producingTransports[trans]
            if not self.producingTransports:
                self.transport.unregisterProducer()


class SubProducer:
    """ I am a mixin that provides upwards-registration of my producer to a
    SuperProducer instance.
    """
    def __init__(self, superproducer):
        self.superproducer = superproducer
        self.producer = None
        self.parentAcceptingData = True
        self.peerAcceptingData = True
        self.producerPaused = False
        self.parentStopped = False

    def maybeResumeProducing(self):
        if ((self.producer is not None) and
            ((not self.streamingProducer) or
             (self.producerPaused)) and
            (self.peerAcceptingData) and
            (self.parentAcceptingData)):
            self.producerPaused = False
            self.producer.resumeProducing()

    def maybePauseProducing(self):
        if ((self.producer is not None) and
            ((not self.peerAcceptingData) or
             (not self.parentAcceptingData)) and
            (not self.producerPaused)):
            self.producerPaused = True
            self.producer.pauseProducing()

    def parentResumeProducing(self):
        self.parentAcceptingData = True
        self.maybeResumeProducing()

    def parentPauseProducing(self):
        self.parentAcceptingData = False
        self.maybePauseProducing()

    def parentStopProducing(self):
        self.parentStopped = True
        if self.producer is not None:
            self.producer.stopProducing()

    def choke(self):
        self.peerAcceptingData = False
        self.maybePauseProducing()

    def unchoke(self):
        self.peerAcceptingData = True
        self.maybeResumeProducing()

    def registerProducer(self, producer, streaming):
        if self.parentStopped:
            producer.stopProducing()
            return
        self.producer = producer
        self.streamingProducer = streaming
        self.superproducer.registerProducerFor(self)

    def unregisterProducer(self):
        if not self.parentStopped:
            self.superproducer.unregisterProducerFor(self)
        self.producer = None





from twisted.trial import unittest

from vertex import statemachine

S_1, S_2 = 'S_1', 'S_2'
I_1, I_2 = 'I_1', 'I_2'
O_1, O_2 = 'O_1', 'O_2'

class TestMachine(statemachine.StateMachine):
    states = {
        S_1: {
            I_1: (O_1, S_1),
            I_2: (O_1, S_2),
            },
        S_2: {
            I_1: (O_2, S_2),
            I_2: (O_1, S_2),
            },
        }

    initialState = S_1

    def _event(self, name):
        def method():
            self.events.append(name)
        return method

    def __getattr__(self, name):
        if (name.startswith('exit_') or name.startswith('enter_') or
            name.startswith('transition_') or name.startswith('output_')):
            return self._event(name)
        raise AttributeError(name)

class Transitions(unittest.TestCase):
    def testOutput(self):
        m = TestMachine()

        self.assertEquals(m.state, S_1)

        m.events = []
        m.input(I_1)
        self.assertEquals(
            m.events,
            ['output_O_1'])
        self.assertEquals(m.state, S_1)

        m.events = []
        m.input(I_2)
        self.assertEquals(
            m.events,
            ['exit_S_1', 'enter_S_2', 'transition_S_1_to_S_2', 'output_O_1'])
        self.assertEquals(m.state, S_2)

        m.events = []
        m.input(I_1)
        self.assertEquals(
            m.events,
            ['output_O_2'])
        self.assertEquals(m.state, S_2)

        m.events = []
        m.input(I_2)
        self.assertEquals(
            m.events,
            ['output_O_1'])
        self.assertEquals(m.state, S_2)



# Copyright 2005 Divmod, Inc.  See LICENSE file for details

"""
This module is no longer supported for use outside Vertex.
"""

from twisted.python import log
from sets import Set
from twisted.persisted import sob
from twisted.application import service, internet

from zope.interface import implements

class Conf(dict):
    """A class to help in construction the configuration for delpoy().

    Typical usage:

    from vertex.depserv import Conf
    conf = Conf()
    s = conf.section
    s('pop',
        port = 110,
        sslPort = 995)
    ...
    """
    def section(self, name, **kw):
        self.setdefault(name, {}).update(kw)


class NotPersistable:
    implements(sob.IPersistable)
    def __init__(self, original):
        self.original = original

    def setStyle(self, style):
        self.style = style

    def save(self, tag=None, filename=None, passphrase=None):
        pass


class StartupError(Exception):
    pass


class DependencyService(service.MultiService):
    """A MultiService that can start multiple services with interdependencies.

    Each keyword parameter is a dict which serves as the options for that
    service.

    Each service defines a method setup_SERVICE, which is called with the
    matching parameters (the service name must be all caps). If there is no key
    for SERVICE in the class parameters, the setup method is not called. The
    return value is ignored, and DependencyService makes no assumptions about
    any side effects.

    Each service may also optionally define depends_SERVICE which is called
    before the setup method with the same parameters as the setup method. This
    method returns a list of names of services on which SERVICE depends.
    DependencyService will then initialize the service is the correct order. If
    circular dependencies result, or a service depends on another service which
    does not exist or is not configured to run, StartupError is raised.

    The class can define required services by setting 'requiredServices' to a
    list of service names. These services will be initialized first in the
    order they appear in the list, ignoring all dependency information. If
    there are no parameters for a required service (consequently, the setup
    method would not normally be called), StartupError is raised.
    """


    requiredServices = []


    def __init__(self, **kw):
        service.MultiService.__init__(self)

        # this makes it possible for one service to change the configuration of
        # another. Avoid if possible, there if you need it. Be sure to properly
        # set the dependencies.
        self.config = kw
        self.servers = []

        services = kw.keys()
        initedServices = Set()
        uninitedServices = Set(services)

        # build dependencies
        dependencies = {}
        for serv in services:
            try:
                dependMethod = self._getDependsMethod(serv)
            except AttributeError:
                continue
            dependencies[serv] = dependMethod(**kw[serv])

        def initializeService(svc):
            self._getServiceMethod(svc)(**kw[svc])
            initedServices.add(svc)
            uninitedServices.remove(svc)

        for svc in self.requiredServices:
            if dependencies.get(svc):
                raise StartupError(
                    '%r is a required service but has unsatisfied '
                    'dependency on %r' % (svc, dependencies[svc]))
            initializeService(svc)

        while uninitedServices:
            # iterate over the uninitialized services, adding those with no
            # outstanding dependencies to initThisRound.
            initThisRound = []
            for serv in uninitedServices:
                for dep in dependencies.get(serv, []):
                    if dep not in initedServices:
                        if dep not in uninitedServices:
                            raise StartupError(
                                'service %r depends on service %r, which is not '
                                'configured or does not exist.' % (serv, dep))
                        break
                else:
                    initThisRound.append(serv)
            if not initThisRound:
                raise StartupError(
                    'Can not initialize all services. Circular dependencies '
                    'between setup methods?')
            for svc in initThisRound:
                initializeService(svc)


    def _getServiceMethod(self, service):
        return getattr(self, 'setup_%s' % (service.upper(),))


    def _getDependsMethod(self, service):
        return getattr(self, 'depends_%s' % (service.upper(),))


    def deploy(Class, name=None, uid=None, gid=None, **kw):
        """Create an application with the give name, uid, and gid.

        The application has one child service, an instance of Class
        configured based on the additional keyword arguments passed.

        The application is not persistable.
        """
        svc = Class(**kw)

        if name is None:
            name = Class.__name__
        # Make it easier (possible) to find this service by name later on
        svc.setName(name)

        app = service.Application(name, uid=uid, gid=gid)
        app.addComponent(NotPersistable(app), ignoreClass=True)
        svc.setServiceParent(app)

        return app
    deploy = classmethod(deploy)

    def attach(self, subservice):
        subservice.setServiceParent(self)
        return subservice

    def detach(self, subservice):
        subservice.disownServiceParent()

    def addServer(self, normalPort, sslPort, f, name):
        """Add a TCP and an SSL server. Name them `name` and `name`+'s'."""
        tcp = internet.TCPServer(normalPort,f)
        tcp.setName(name)
        self.servers.append(tcp)
        if sslPort is not None:
            ssl = internet.SSLServer(sslPort, f, contextFactory=self.sslfac)
            ssl.setName(name+'s')
            self.servers.append(ssl)

    def discernPrivilegedServers(self):
        return [srv for srv in self.servers if srv.args[0] <= 1024]

    def discernUnprivilegedServers(self):
        return [srv for srv in self.servers if srv.args[0] > 1024]

    def privilegedStartService(self):
        for server in self.discernPrivilegedServers():
            log.msg("privileged attach %r" % server)
            self.attach(server)
        return service.MultiService.privilegedStartService(self)

    def startService(self):
        for server in self.discernUnprivilegedServers():
            log.msg("attaching %r" % server)
            self.attach(server)

        return service.MultiService.startService(self)



# Copyright 2005 Divmod, Inc.  See LICENSE file for details
# -*- test-case-name: vertex.test.test_sslverify -*-

import itertools, md5
from OpenSSL import SSL, crypto

from twisted.python import reflect
from twisted.internet.defer import Deferred

# Private - shared between all ServerContextFactories, counts up to
# provide a unique session id for each context
_sessionCounter = itertools.count().next

class _SSLApplicationData(object):
    def __init__(self):
        self.problems = []

class VerifyError(Exception):
    """Could not verify something that was supposed to be signed.
    """

class PeerVerifyError(VerifyError):
    """The peer rejected our verify error.
    """

class OpenSSLVerifyError(VerifyError):

    _errorCodes = {0: ('X509_V_OK', 'ok', 'the operation was successful. >'),
                   2: ('X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT',
                       'unable to get issuer certificate',
                       "the issuer certificate could not be found', 'this occurs if the issuer certificate of an untrusted certificate cannot be found."),
                   3: ('X509_V_ERR_UNABLE_TO_GET_CRL',
                       ' unable to get certificate CRL',
                       'the CRL of a certificate could not be found. Unused.'),
                   4: ('X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE',
                       "unable to decrypt certificate's signature",
                       'the certificate signature could not be decrypted. This means that the actual signature value could not be determined rather than it not match- ing the expected value, this is only meaningful for RSA keys.'),
                   5: ('X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE',
                       "unable to decrypt CRL's signature",
                       "the CRL signature could not be decrypted', 'this means that the actual signature value could not be determined rather than it not matching the expected value. Unused."),
                   6: ('X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY',
                       'unable to decode issuer',
                       'public key the public key in the certificate SubjectPublicKeyInfo could not be read.'),
                   7: ('X509_V_ERR_CERT_SIGNATURE_FAILURE',
                       'certificate signature failure',
                       'the signature of the certificate is invalid.'),
                   8: ('X509_V_ERR_CRL_SIGNATURE_FAILURE',
                       'CRL signature failure',
                       'the signature of the certificate is invalid. Unused.'),
                   9: ('X509_V_ERR_CERT_NOT_YET_VALID',
                       'certificate is not yet valid',
                       "the certificate is not yet valid', 'the notBefore date is after the cur- rent time."),
                   10: ('X509_V_ERR_CERT_HAS_EXPIRED',
                        'certificate has expired',
                        "the certificate has expired', 'that is the notAfter date is before the current time."),
                   11: ('X509_V_ERR_CRL_NOT_YET_VALID',
                        'CRL is not yet valid',
                        'the CRL is not yet valid. Unused.'),
                   12: ('X509_V_ERR_CRL_HAS_EXPIRED',
                        'CRL has expired',
                        'the CRL has expired. Unused.'),
                   13: ('X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD',
                        "format error in certificate's",
                        'notBefore field the certificate notBefore field contains an invalid time.'),
                   14: ('X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD',
                        "format error in certificate's",
                        'notAfter field the certificate notAfter field contains an invalid time.'),
                   15: ('X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD',
                        "format error in CRL's lastUpdate field",
                        'the CRL lastUpdate field contains an invalid time. Unused.'),
                   16: ('X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD',
                        "format error in CRL's nextUpdate field",
                        'the CRL nextUpdate field contains an invalid time. Unused.'),
                   17: ('X509_V_ERR_OUT_OF_MEM',
                        'out of memory',
                        'an error occurred trying to allocate memory. This should never happen.'),
                   18: ('X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT',
                        'self signed certificate',
                        'the passed certificate is self signed and the same certificate cannot be found in the list of trusted certificates.'),
                   19: ('X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN',
                        'self signed certificate in certificate chain',
                        'the certificate chain could be built up using the untrusted certificates but the root could not be found locally.'),
                   20: ('X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY',
                        'unable to get local issuer',
                        'certificate the issuer certificate of a locally looked up certificate could not be found. This normally means the list of trusted certificates is not com- plete.'),
                   21: ('X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE',
                        'unable to verify the first',
                        'certificate no signatures could be verified because the chain contains only one cer- tificate and it is not self signed.'),
                   22: ('X509_V_ERR_CERT_CHAIN_TOO_LONG',
                        'certificate chain too long',
                        'the certificate chain length is greater than the supplied maximum depth. Unused.'),
                   23: ('X509_V_ERR_CERT_REVOKED',
                        'certificate revoked',
                        'the certificate has been revoked. Unused.'),
                   24: ('X509_V_ERR_INVALID_CA',
                        'invalid CA certificate',
                        'a CA certificate is invalid. Either it is not a CA or its extensions are not consistent with the supplied purpose.'),
                   25: ('X509_V_ERR_PATH_LENGTH_EXCEEDED',
                        'path length constraint exceeded',
                        'the basicConstraints pathlength parameter has been exceeded.'),
                   26: ('X509_V_ERR_INVALID_PURPOSE',
                        'unsupported certificate purpose',
                        'the supplied certificate cannot be used for the specified purpose.'),
                   27: ('X509_V_ERR_CERT_UNTRUSTED',
                        'certificate not trusted',
                        'the root CA is not marked as trusted for the specified purpose.'),
                   28: ('X509_V_ERR_CERT_REJECTED',
                        'certificate rejected',
                        'the root CA is marked to reject the specified purpose.'),
                   29: ('X509_V_ERR_SUBJECT_ISSUER_MISMATCH',
                        'subject issuer mismatch',
                        'the current candidate issuer certificate was rejected because its sub- ject name did not match the issuer name of the current certificate. Only displayed when the -issuer_checks option is set.'),
                   30: ('X509_V_ERR_AKID_SKID_MISMATCH',
                        'authority and subject key identifier mismatch',
                        'the current candidate issuer certificate was rejected because its sub- ject key identifier was present and did not match the authority key identifier current certificate. Only displayed when the -issuer_checks option is set.'),
                   31: ('X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH',
                        'authority and issuer serial number mismatch',
                        'the current candidate issuer certificate was rejected because its issuer name and serial number was present and did not match the authority key identifier of the current certificate. Only displayed when the -issuer_checks option is set.'),
                   32: ('X509_V_ERR_KEYUSAGE_NO_CERTSIGN',
                        'key usage does not include certificate',
                        'signing the current candidate issuer certificate was rejected because its keyUsage extension does not permit certificate signing.'),
                   50: ('X509_V_ERR_APPLICATION_VERIFICATION',
                        'application verification failure',
                        'an application specific error. Unused.')}


    def __init__(self, cert, errno, depth):
        VerifyError.__init__(self, cert, errno, depth)
        self.cert = cert
        self.errno = errno
        self.depth = depth

    def __repr__(self):
        x = self._errorCodes.get(self.errno)
        if x is not None:
            name, short, long = x
            return 'Peer Certificate Verification Failed: %s (error code: %d)' % (
                long, self.errno
                )

    __str__ = __repr__

_x509namecrap = [
    ['CN', 'commonName'],
    ['O',  'organizationName'],
    ['OU', 'organizationalUnitName'],
    ['L',  'localityName'],
    ['ST', 'stateOrProvinceName'],
    ['C',  'countryName'],
    [      'emailAddress']]

_x509names = {}


for abbrevs in _x509namecrap:
    for abbrev in abbrevs:
        _x509names[abbrev] = abbrevs[0]

class DistinguishedName(dict):
    __slots__ = ()
    def __init__(self, **kw):
        for k, v in kw.iteritems():
            setattr(self, k, v)

    def _copyFrom(self, x509name):
        d = {}
        for name in _x509names:
            value = getattr(x509name, name, None)
            if value is not None:
                setattr(self, name, value)

    def _copyInto(self, x509name):
        for k, v in self.iteritems():
            setattr(x509name, k, v)

    def __repr__(self):
        return '<DN %s>' % (dict.__repr__(self)[1:-1])

    def __getattr__(self, attr):
        return self[_x509names[attr]]

    def __setattr__(self, attr, value):
        assert type(attr) is str
        if not attr in _x509names:
            raise AttributeError("%s is not a valid OpenSSL X509 name field" % (attr,))
        realAttr = _x509names[attr]
        value = value.encode('ascii')
        assert type(value) is str
        self[realAttr] = value

    def inspect(self):
        l = []
        from formless.annotate import nameToLabel
        lablen = 0
        for kp in _x509namecrap:
            k = kp[-1]
            label = nameToLabel(k)
            lablen = max(len(label), lablen)
            l.append((label, getattr(self, k)))
        lablen += 2
        for n, (label, attr) in enumerate(l):
            l[n] = (label.rjust(lablen)+': '+ attr)
        return '\n'.join(l)

DN = DistinguishedName

class CertBase:
    def __init__(self, original):
        self.original = original

    def _copyName(self, suffix):
        dn = DistinguishedName()
        dn._copyFrom(getattr(self.original, 'get_'+suffix)())
        return dn

    def getSubject(self):
        return self._copyName('subject')



def problemsFromTransport(tpt):
    """Return a list of L{OpenSSLVerifyError}s given a Twisted transport object.
    """
    return tpt.getHandle().get_context().get_app_data().problems

class Certificate(CertBase):
    def __repr__(self):
        return '<%s Subject=%s Issuer=%s>' % (self.__class__.__name__,
                                              self.getSubject().commonName,
                                              self.getIssuer().commonName)

    def __eq__(self, other):
        if isinstance(other, Certificate):
            return self.dump() == other.dump()
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def load(Class, requestData, format=crypto.FILETYPE_ASN1, args=()):
        return Class(crypto.load_certificate(format, requestData), *args)
    load = classmethod(load)
    _load = load

    def dumpPEM(self):
        """Dump both public and private parts of a private certificate to PEM-format
        data
        """
        return self.dump(crypto.FILETYPE_PEM)

    def loadPEM(Class, data):
        """Load both private and public parts of a private certificate from a chunk of
        PEM-format data.
        """
        return Class.load(data, crypto.FILETYPE_PEM)
    loadPEM = classmethod(loadPEM)

    def peerFromTransport(Class, transport):
        return Class(transport.getHandle().get_peer_certificate())
    peerFromTransport = classmethod(peerFromTransport)

    def hostFromTransport(Class, transport):
        return Class(transport.getHandle().get_host_certificate())
    hostFromTransport = classmethod(hostFromTransport)

    def getPublicKey(self):
        return PublicKey(self.original.get_pubkey())

    def dump(self, format=crypto.FILETYPE_ASN1):
        return crypto.dump_certificate(format, self.original)

    def serialNumber(self):
        return self.original.get_serial_number()

    def digest(self, method='md5'):
        return self.original.digest(method)

    def _inspect(self):
        return '\n'.join(['Certificate For Subject:',
                          self.getSubject().inspect(),
                          '\nIssuer:',
                          self.getIssuer().inspect(),
                          '\nSerial Number: %d' % self.serialNumber(),
                          'Digest: %s' % self.digest()])

    def inspect(self):
        return '\n'.join(self._inspect(), self.getPublicKey().inspect())

    def getIssuer(self):
        return self._copyName('issuer')

    def options(self, *authorities):
        raise NotImplementedError('Possible, but doubtful we need this yet')

class CertificateRequest(CertBase):
    def load(Class, requestData, requestFormat=crypto.FILETYPE_ASN1):
        req = crypto.load_certificate_request(requestFormat, requestData)
        dn = DistinguishedName()
        dn._copyFrom(req.get_subject())
        if not req.verify(req.get_pubkey()):
            raise VerifyError("Can't verify that request for %r is self-signed." % (dn,))
        return Class(req)
    load = classmethod(load)

    def dump(self, format=crypto.FILETYPE_ASN1):
        return crypto.dump_certificate_request(format, self.original)

class PrivateCertificate(Certificate):
    def __repr__(self):
        return Certificate.__repr__(self) + ' with ' + repr(self.privateKey)

    def _setPrivateKey(self, privateKey):
        if not privateKey.matches(self.getPublicKey()):
            raise VerifyError(
                "Sanity check failed: Your certificate was not properly signed.")
        self.privateKey = privateKey
        return self

    def newCertificate(self, newCertData, format=crypto.FILETYPE_ASN1):
        return self.load(newCertData, self.privateKey, format)

    def load(Class, data, privateKey, format=crypto.FILETYPE_ASN1):
        return Class._load(data, format)._setPrivateKey(privateKey)
    load = classmethod(load)

    def inspect(self):
        return '\n'.join([Certificate._inspect(self),
                          self.privateKey.inspect()])

    def dumpPEM(self):
        """Dump both public and private parts of a private certificate to PEM-format
        data
        """
        return self.dump(crypto.FILETYPE_PEM) + self.privateKey.dump(crypto.FILETYPE_PEM)

    def loadPEM(Class, data):
        """Load both private and public parts of a private certificate from a chunk of
        PEM-format data.
        """
        return Class.load(data, KeyPair.load(data, crypto.FILETYPE_PEM),
                          crypto.FILETYPE_PEM)

    loadPEM = classmethod(loadPEM)

    def fromCertificateAndKeyPair(Class, certificateInstance, privateKey):
        privcert = Class(certificateInstance.original)
        return privcert._setPrivateKey(privateKey)
    fromCertificateAndKeyPair = classmethod(fromCertificateAndKeyPair)

    def options(self, *authorities):
        options = dict(privateKey=self.privateKey.original,
                       certificate=self.original)
        if authorities:
            options.update(dict(verify=True,
                                requireCertificate=True,
                                caCerts=[auth.original for auth in authorities]))
        return OpenSSLCertificateOptions(**options)

    def certificateRequest(self, format=crypto.FILETYPE_ASN1,
                           digestAlgorithm='md5'):
        return self.privateKey.certificateRequest(
            self.getSubject(),
            format,
            digestAlgorithm)

    def signCertificateRequest(self,
                               requestData,
                               verifyDNCallback,
                               serialNumber,
                               requestFormat=crypto.FILETYPE_ASN1,
                               certificateFormat=crypto.FILETYPE_ASN1):
        issuer = self.getSubject()
        return self.privateKey.signCertificateRequest(
            issuer,
            requestData,
            verifyDNCallback,
            serialNumber,
            requestFormat,
            certificateFormat)


    def signRequestObject(self, certificateRequest, serialNumber,
                          secondsToExpiry=60 * 60 * 24 * 365, # One year
                          digestAlgorithm='md5'):
        return self.privateKey.signRequestObject(self.getSubject(),
                                                 certificateRequest,
                                                 serialNumber,
                                                 secondsToExpiry,
                                                 digestAlgorithm)

class PublicKey:
    def __init__(self, osslpkey):
        self.original = osslpkey
        req1 = crypto.X509Req()
        req1.set_pubkey(osslpkey)
        self._emptyReq = crypto.dump_certificate_request(crypto.FILETYPE_ASN1, req1)

    def matches(self, otherKey):
        return self._emptyReq == otherKey._emptyReq

# O OG OMG OMFG PYOPENSSL SUCKS SO BAD
#     def verifyCertificate(self, certificate):
#         """returns None, or raises a VerifyError exception if the certificate could not
#         be verified.
#         """
#         if not certificate.original.verify(self.original):
#             raise VerifyError("We didn't sign that certificate.")

    def __repr__(self):
        return '<%s %s>' % (self.__class__.__name__, self.keyHash())

    def keyHash(self):
        """MD5 hex digest of signature on an empty certificate request with this key.
        """
        return md5.md5(self._emptyReq).hexdigest()

    def inspect(self):
        return 'Public Key with Hash: %s' % (self.keyHash(),)


class KeyPair(PublicKey):

    def load(Class, data, format=crypto.FILETYPE_ASN1):
        return Class(crypto.load_privatekey(format, data))
    load = classmethod(load)

    def dump(self, format=crypto.FILETYPE_ASN1):
        return crypto.dump_privatekey(format, self.original)

    def __getstate__(self):
        return self.dump()

    def __setstate__(self, state):
        self.__init__(crypto.load_privatekey(crypto.FILETYPE_ASN1, state))

    def inspect(self):
        t = self.original.type()
        if t == crypto.TYPE_RSA:
            ts = 'RSA'
        elif t == crypto.TYPE_DSA:
            ts = 'DSA'
        else:
            ts = '(Unknown Type!)'
        L = (self.original.bits(), ts, self.keyHash())
        return '%s-bit %s Key Pair with Hash: %s' % L

    def generate(Class, kind=crypto.TYPE_RSA, size=1024):
        pkey = crypto.PKey()
        pkey.generate_key(kind, size)
        return Class(pkey)

    def newCertificate(self, newCertData, format=crypto.FILETYPE_ASN1):
        return PrivateCertificate.load(newCertData, self, format)

    generate = classmethod(generate)

    def requestObject(self, distinguishedName, digestAlgorithm='md5'):
        req = crypto.X509Req()
        req.set_pubkey(self.original)
        distinguishedName._copyInto(req.get_subject())
        req.sign(self.original, digestAlgorithm)
        return CertificateRequest(req)

    def certificateRequest(self, distinguishedName,
                           format=crypto.FILETYPE_ASN1,
                           digestAlgorithm='md5'):
        """Create a certificate request signed with this key.

        @return: a string, formatted according to the 'format' argument.
        """
        return self.requestObject(distinguishedName, digestAlgorithm).dump(format)



    def signCertificateRequest(self,
                               issuerDistinguishedName,
                               requestData,
                               verifyDNCallback,
                               serialNumber,
                               requestFormat=crypto.FILETYPE_ASN1,
                               certificateFormat=crypto.FILETYPE_ASN1,
                               secondsToExpiry=60 * 60 * 24 * 365, # One year
                               digestAlgorithm='md5'):
        """Given a blob of certificate request data and a certificate authority's
        DistinguishedName, return a blob of signed certificate data.

        If verifyDNCallback returns a Deferred, I will return a Deferred which
        fires the data when that Deferred has completed.
        """
        hlreq = CertificateRequest.load(requestData, requestFormat)

        dn = hlreq.getSubject()
        vval = verifyDNCallback(dn)

        def verified(value):
            if not value:
                raise VerifyError("DN callback %r rejected request DN %r" % (verifyDNCallback, dn))
            return self.signRequestObject(issuerDistinguishedName, hlreq,
                                          serialNumber, secondsToExpiry, digestAlgorithm).dump(certificateFormat)

        if isinstance(vval, Deferred):
            return vval.addCallback(verified)
        else:
            return verified(vval)


    def signRequestObject(self,
                          issuerDistinguishedName,
                          requestObject,
                          serialNumber,
                          secondsToExpiry=60 * 60 * 24 * 365, # One year
                          digestAlgorithm='md5'):
        """
        Sign a CertificateRequest instance, returning a Certificate instance.
        """
        req = requestObject.original
        dn = requestObject.getSubject()
        cert = crypto.X509()
        issuerDistinguishedName._copyInto(cert.get_issuer())
        cert.set_subject(req.get_subject())
        cert.set_pubkey(req.get_pubkey())
        cert.gmtime_adj_notBefore(0)
        cert.gmtime_adj_notAfter(secondsToExpiry)
        cert.set_serial_number(serialNumber)
        cert.sign(self.original, digestAlgorithm)
        return Certificate(cert)

    def selfSignedCert(self, serialNumber, **kw):
        dn = DN(**kw)
        return PrivateCertificate.fromCertificateAndKeyPair(
            self.signRequestObject(dn, self.requestObject(dn), serialNumber),
            self)


class OpenSSLCertificateOptions(object):
    """A factory for SSL context objects, for server SSL connections.
    """

    _context = None
    # Older versions of PyOpenSSL didn't provide OP_ALL.  Fudge it here, just in case.
    _OP_ALL = getattr(SSL, 'OP_ALL', 0x0000FFFF)

    method = SSL.TLSv1_METHOD

    def __init__(self,
                 privateKey=None,
                 certificate=None,
                 method=None,
                 verify=False,
                 caCerts=None,
                 verifyDepth=9,
                 requireCertificate=True,
                 verifyOnce=True,
                 enableSingleUseKeys=True,
                 enableSessions=True,
                 fixBrokenPeers=True):
        """
        Create an OpenSSL context SSL connection context factory.

        @param privateKey: A PKey object holding the private key.

        @param certificate: An X509 object holding the certificate.

        @param method: The SSL protocol to use, one of SSLv23_METHOD,
        SSLv2_METHOD, SSLv3_METHOD, TLSv1_METHOD.  Defaults to TLSv1_METHOD.

        @param verify: If True, verify certificates received from the peer and
        fail the handshake if verification fails.  Otherwise, allow anonymous
        sessions and sessions with certificates which fail validation.  By
        default this is False.

        @param caCerts: List of certificate authority certificates to
        send to the client when requesting a certificate.  Only used if verify
        is True, and if verify is True, either this must be specified or
        caCertsFile must be given.  Since verify is False by default,
        this is None by default.

        @param verifyDepth: Depth in certificate chain down to which to verify.
        If unspecified, use the underlying default (9).

        @param requireCertificate: If True, do not allow anonymous sessions.

        @param verifyOnce: If True, do not re-verify the certificate
        on session resumption.

        @param enableSingleUseKeys: If True, generate a new key whenever
        ephemeral DH parameters are used to prevent small subgroup attacks.

        @param enableSessions: If True, set a session ID on each context.  This
        allows a shortened handshake to be used when a known client reconnects.

        @param fixBrokenPeers: If True, enable various non-spec protocol fixes
        for broken SSL implementations.  This should be entirely safe,
        according to the OpenSSL documentation, but YMMV.
        """
        assert (privateKey is None) == (certificate is None), "Specify neither or both of privateKey and certificate"
        self.privateKey = privateKey
        self.certificate = certificate
        if method is not None:
            self.method = method

        self.verify = verify
        assert ((verify and caCerts) or
                (not verify)), "Specify client CA certificate information if and only if enabling certificate verification"

        self.caCerts = caCerts
        self.verifyDepth = verifyDepth
        self.requireCertificate = requireCertificate
        self.verifyOnce = verifyOnce
        self.enableSingleUseKeys = enableSingleUseKeys
        self.enableSessions = enableSessions
        self.fixBrokenPeers = True

    def __getstate__(self):
        d = super(OpenSSLCertificateOptions, self).__getstate__()
        try:
            del d['context']
        except KeyError:
            pass
        return d

    def getContext(self):
        """Return a SSL.Context object.
        """
        if self._context is None:
            self._context = self._makeContext()
        return self._context

    def _makeContext(self):
        ctx = SSL.Context(self.method)
        ctx.set_app_data(_SSLApplicationData())

        if self.certificate is not None and self.privateKey is not None:
            ctx.use_certificate(self.certificate)
            ctx.use_privatekey(self.privateKey)
            # Sanity check
            ctx.check_privatekey()

        verifyFlags = SSL.VERIFY_NONE
        if self.verify:
            verifyFlags = SSL.VERIFY_PEER
            if self.requireCertificate:
                verifyFlags |= SSL.VERIFY_FAIL_IF_NO_PEER_CERT
            if self.verifyOnce:
                verifyFlags |= SSL.VERIFY_CLIENT_ONCE
            if self.caCerts:
                store = ctx.get_cert_store()
                for cert in self.caCerts:
                    store.add_cert(cert)

        def _trackVerificationProblems(conn,cert,errno,depth,preverify_ok):
            # retcode is the answer OpenSSL's default verifier would have
            # given, had we allowed it to run.
            if not preverify_ok:
                ctx.get_app_data().problems.append(OpenSSLVerifyError(cert, errno, depth))
            return preverify_ok
        ctx.set_verify(verifyFlags, _trackVerificationProblems)

        if self.verifyDepth is not None:
            ctx.set_verify_depth(self.verifyDepth)

        if self.enableSingleUseKeys:
            ctx.set_options(SSL.OP_SINGLE_DH_USE)

        if self.fixBrokenPeers:
            ctx.set_options(self._OP_ALL)

        if self.enableSessions:
            sessionName = md5.md5("%s-%d" % (reflect.qual(self.__class__), _sessionCounter())).hexdigest()
            ctx.set_session_id(sessionName)

        return ctx



# Copyright 2005 Divmod, Inc.  See LICENSE file for details


from vertex import juice
from vertex.test import iosim
from twisted.trial import unittest
from twisted.internet import protocol, defer

class TestProto(protocol.Protocol):
    def __init__(self, onConnLost, dataToSend):
        self.onConnLost = onConnLost
        self.dataToSend = dataToSend

    def connectionMade(self):
        self.data = []
        self.transport.write(self.dataToSend)

    def dataReceived(self, bytes):
        self.data.append(bytes)
        self.transport.loseConnection()

    def connectionLost(self, reason):
        self.onConnLost.callback(self.data)

class SimpleSymmetricProtocol(juice.Juice):

    def sendHello(self, text):
        return self.sendCommand("hello",
                                hello=text)

    def sendGoodbye(self):
        return self.sendCommand("goodbye")

    def juice_HELLO(self, box):
        return juice.Box(hello=box['hello'])

    def juice_GOODBYE(self, box):
        return juice.QuitBox(goodbye='world')

class UnfriendlyGreeting(Exception):
    """Greeting was insufficiently kind.
    """

class UnknownProtocol(Exception):
    """Asked to switch to the wrong protocol.
    """

class Hello(juice.Command):
    commandName = 'hello'
    arguments = [('hello', juice.String())]
    response = [('hello', juice.String())]

    errors = {UnfriendlyGreeting: 'UNFRIENDLY'}

class Goodbye(juice.Command):
    commandName = 'goodbye'
    responseType = juice.QuitBox

class GetList(juice.Command):
    commandName = 'getlist'
    arguments = [('length', juice.Integer())]
    response = [('body', juice.JuiceList([('x', juice.Integer())]))]

class TestSwitchProto(juice.ProtocolSwitchCommand):
    commandName = 'Switch-Proto'

    arguments = [
        ('name', juice.String()),
        ]
    errors = {UnknownProtocol: 'UNKNOWN'}

class SingleUseFactory(protocol.ClientFactory):
    def __init__(self, proto):
        self.proto = proto

    def buildProtocol(self, addr):
        p, self.proto = self.proto, None
        return p

class SimpleSymmetricCommandProtocol(juice.Juice):
    maybeLater = None
    def __init__(self, issueGreeting, onConnLost=None):
        juice.Juice.__init__(self, issueGreeting)
        self.onConnLost = onConnLost

    def sendHello(self, text):
        return Hello(hello=text).do(self)
    def sendGoodbye(self):
        return Goodbye().do(self)
    def command_HELLO(self, hello):
        if hello.startswith('fuck'):
            raise UnfriendlyGreeting("Don't be a dick.")
        return dict(hello=hello)
    def command_GETLIST(self, length):
        return {'body': [dict(x=1)] * length}
    def command_GOODBYE(self):
        return dict(goodbye='world')
    command_HELLO.command = Hello
    command_GOODBYE.command = Goodbye
    command_GETLIST.command = GetList

    def switchToTestProtocol(self):
        p = TestProto(self.onConnLost, SWITCH_CLIENT_DATA)
        return TestSwitchProto(SingleUseFactory(p), name='test-proto').do(self).addCallback(lambda ign: p)

    def command_SWITCH_PROTO(self, name):
        if name == 'test-proto':
            return TestProto(self.onConnLost, SWITCH_SERVER_DATA)
        raise UnknownProtocol(name)

    command_SWITCH_PROTO.command = TestSwitchProto

class DeferredSymmetricCommandProtocol(SimpleSymmetricCommandProtocol):
    def command_SWITCH_PROTO(self, name):
        if name == 'test-proto':
            self.maybeLaterProto = TestProto(self.onConnLost, SWITCH_SERVER_DATA)
            self.maybeLater = defer.Deferred()
            return self.maybeLater
        raise UnknownProtocol(name)

    command_SWITCH_PROTO.command = TestSwitchProto


class SSPF: protocol = SimpleSymmetricProtocol
class SSSF(SSPF, protocol.ServerFactory): pass
class SSCF(SSPF, protocol.ClientFactory): pass

def connectedServerAndClient(ServerClass=lambda: SimpleSymmetricProtocol(True),
                             ClientClass=lambda: SimpleSymmetricProtocol(False),
                             *a, **kw):
    """Returns a 3-tuple: (client, server, pump)
    """
    return iosim.connectedServerAndClient(
        ServerClass, ClientClass,
        *a, **kw)

class TotallyDumbProtocol(protocol.Protocol):
    buf = ''
    def dataReceived(self, data):
        self.buf += data

class LiteralJuice(juice.Juice):
    def __init__(self, issueGreeting):
        juice.Juice.__init__(self, issueGreeting)
        self.boxes = []

    def juiceBoxReceived(self, box):
        self.boxes.append(box)
        return

class LiteralParsingTest(unittest.TestCase):
    def testBasicRequestResponse(self):
        c, s, p = connectedServerAndClient(ClientClass=TotallyDumbProtocol)
        HELLO = 'abcdefg'
        ASKTOK = 'hand-crafted-ask'
        c.transport.write(("""-Command: HeLlO
-Ask: %s
Hello: %s
World: this header is ignored

""" % (ASKTOK, HELLO,)).replace('\n','\r\n'))
        p.flush()
        asserts = {'hello': HELLO,
                   '-answer': ASKTOK}
        hdrs = [j.split(': ') for j in c.buf.split('\r\n')[:-2]]
        self.assertEquals(len(asserts), len(hdrs))
        for hdr in hdrs:
            k, v = hdr
            self.assertEquals(v, asserts[k.lower()])

    def testParsingRoundTrip(self):
        c, s, p = connectedServerAndClient(ClientClass=lambda: LiteralJuice(False),
                                           ServerClass=lambda: LiteralJuice(True))

        SIMPLE = ('simple', 'test')
        CE = ('ceq', ': ')
        CR = ('crtest', 'test\r')
        LF = ('lftest', 'hello\n')
        NEWLINE = ('newline', 'test\r\none\r\ntwo')
        NEWLINE2 = ('newline2', 'test\r\none\r\n two')
        BLANKLINE = ('newline3', 'test\r\n\r\nblank\r\n\r\nline')
        BODYTEST = (juice.BODY, 'blah\r\n\r\ntesttest')

        testData = [
            [SIMPLE],
            [SIMPLE, BODYTEST],
            [SIMPLE, CE],
            [SIMPLE, CR],
            [SIMPLE, CE, CR, LF],
            [CE, CR, LF],
            [SIMPLE, NEWLINE, CE, NEWLINE2],
            [BODYTEST, SIMPLE, NEWLINE]
            ]

        for test in testData:
            jb = juice.Box()
            jb.update(dict(test))
            jb.sendTo(c)
            p.flush()
            self.assertEquals(s.boxes[-1], jb)

SWITCH_CLIENT_DATA = 'Success!'
SWITCH_SERVER_DATA = 'No, really.  Success.'

class AppLevelTest(unittest.TestCase):
    def testHelloWorld(self):
        c, s, p = connectedServerAndClient()
        L = []
        HELLO = 'world'
        c.sendHello(HELLO).addCallback(L.append)
        p.flush()
        self.assertEquals(L[0]['hello'], HELLO)

    def testHelloWorldCommand(self):
        c, s, p = connectedServerAndClient(
            ServerClass=lambda: SimpleSymmetricCommandProtocol(True),
            ClientClass=lambda: SimpleSymmetricCommandProtocol(False))
        L = []
        HELLO = 'world'
        c.sendHello(HELLO).addCallback(L.append)
        p.flush()
        self.assertEquals(L[0]['hello'], HELLO)

    def testHelloErrorHandling(self):
        L=[]
        c, s, p = connectedServerAndClient(ServerClass=lambda: SimpleSymmetricCommandProtocol(True),
                                           ClientClass=lambda: SimpleSymmetricCommandProtocol(False))
        HELLO = 'fuck you'
        c.sendHello(HELLO).addErrback(L.append)
        p.flush()
        L[0].trap(UnfriendlyGreeting)
        self.assertEquals(str(L[0].value), "Don't be a dick.")

    def testJuiceListCommand(self):
        c, s, p = connectedServerAndClient(ServerClass=lambda: SimpleSymmetricCommandProtocol(True),
                                           ClientClass=lambda: SimpleSymmetricCommandProtocol(False))
        L = []
        GetList(length=10).do(c).addCallback(L.append)
        p.flush()
        values = L.pop().get('body')
        self.assertEquals(values, [{'x': 1}] * 10)

    def testFailEarlyOnArgSending(self):
        okayCommand = Hello(Hello="What?")
        self.assertRaises(RuntimeError, Hello)

    def testSupportsVersion1(self):
        c, s, p = connectedServerAndClient(ServerClass=lambda: juice.Juice(True),
                                           ClientClass=lambda: juice.Juice(False))
        negotiatedVersion = []
        s.renegotiateVersion(1).addCallback(negotiatedVersion.append)
        p.flush()
        self.assertEquals(negotiatedVersion[0], 1)
        self.assertEquals(c.protocolVersion, 1)
        self.assertEquals(s.protocolVersion, 1)

    def testProtocolSwitch(self, switcher=SimpleSymmetricCommandProtocol):
        self.testSucceeded = False

        serverDeferred = defer.Deferred()
        serverProto = switcher(True, serverDeferred)
        clientDeferred = defer.Deferred()
        clientProto = switcher(False, clientDeferred)
        c, s, p = connectedServerAndClient(ServerClass=lambda: serverProto,
                                           ClientClass=lambda: clientProto)

        switchDeferred = c.switchToTestProtocol()

        def cbConnsLost(((serverSuccess, serverData), (clientSuccess, clientData))):
            self.failUnless(serverSuccess)
            self.failUnless(clientSuccess)
            self.assertEquals(''.join(serverData), SWITCH_CLIENT_DATA)
            self.assertEquals(''.join(clientData), SWITCH_SERVER_DATA)
            self.testSucceeded = True

        def cbSwitch(proto):
            return defer.DeferredList([serverDeferred, clientDeferred]).addCallback(cbConnsLost)

        switchDeferred.addCallback(cbSwitch)
        p.flush()
        if serverProto.maybeLater is not None:
            serverProto.maybeLater.callback(serverProto.maybeLaterProto)
            p.flush()
        self.failUnless(self.testSucceeded)

    def testProtocolSwitchDeferred(self):
        return self.testProtocolSwitch(switcher=DeferredSymmetricCommandProtocol)




from zope.interface import implements

from twisted.internet import interfaces

from vertex import q2q

class FileWrapper:
    """A wrapper around a file-like object to make it behave as a Transport.

    This doesn't actually stream the file to the attached protocol,
    and is thus useful mainly as a utility for debugging protocols.
    """

    implements(interfaces.ITransport)

    closed = 0
    disconnecting = 0
    disconnected = 0
    producer = None
    streamingProducer = 0

    def __init__(self, file):
        self.file = file

    def write(self, data):
        try:
            self.file.write(data)
        except:
            self.handleException()
        # self._checkProducer()

    def _checkProducer(self):
        # Cheating; this is called at "idle" times to allow producers to be
        # found and dealt with
        if self.producer:
            self.producer.resumeProducing()

    def registerProducer(self, producer, streaming):
        """From abstract.FileDescriptor
        """
        self.producer = producer
        self.streamingProducer = streaming
        if not streaming:
            producer.resumeProducing()

    def unregisterProducer(self):
        self.producer = None

    def stopConsuming(self):
        self.unregisterProducer()
        self.loseConnection()

    def writeSequence(self, iovec):
        self.write("".join(iovec))

    def loseConnection(self):
        self.disconnecting = True

    def getPeer(self):
        # XXX: According to ITransport, this should return an IAddress!
        return 'file', 'file'

    def getHost(self):
        # XXX: According to ITransport, this should return an IAddress!
        return 'file'

    def handleException(self):
        pass

    def resumeProducing(self):
        # Never sends data anyways
        pass

    def pauseProducing(self):
        # Never sends data anyways
        pass

    def stopProducing(self):
        self.loseConnection()

    # Additional Q2Q Transport requirements
    def getQ2QPeer(self):
        return q2q.Q2QAddress('file.domain', 'peer.resource')




# Copyright 2005 Divmod, Inc.  See LICENSE file for details
from twisted.trial import unittest

from vertex import depserv


class Serv(depserv.DependencyService):
    requiredServices = ['one']

    def __init__(self, **kw):
        self.initialized = []
        depserv.DependencyService.__init__(self, **kw)

    def setup_ONE(self):
        self.initialized.append('ONE')

    def setup_TWO(self):
        self.initialized.append('TWO')

    def setup_THREE(self):
        self.initialized.append('THREE')


class TestDependencyService(unittest.TestCase):

    def test_depends(self):
        class One(Serv):
            def depends_TWO(self):
                return ['three']

        class Two(Serv):
            def depends_THREE(self):
                return ['two']

        args = dict(one={}, two={}, three={})

        one = One(**args)
        self.assert_(one.initialized == ['ONE', 'THREE', 'TWO'])

        two = Two(**args)
        self.assert_(two.initialized == ['ONE', 'TWO', 'THREE'])


    def test_circularDepends(self):
        class One(Serv):
            def depends_THREE(self):
                return ['two']
            def depends_TWO(self):
                return ['three']
        try:
            One(one={}, two={}, three={})
        except depserv.StartupError:
            pass
        else:
            raise unittest.FailTest, 'circular dependencies did not raise an error'


    def test_requiredWithDependency(self):
        """A service is required but has dependencies"""

        class One(Serv):
            def depends_ONE(self):
                return ['three']
        try:
            One(one={}, two={}, three={})
        except depserv.StartupError:
            pass
        else:
            raise unittest.FailTest, 'unsatisfied dependencies did not raise an error'



# Copyright 2005 Divmod, Inc.  See LICENSE file for details

import os, md5

from twisted.internet import defer
from vertex import juice, q2q
from vertex.depserv import DependencyService, Conf
from vertex.q2qadmin import AddUser, NotAllowed

class IdentityAdmin(juice.Juice):

    def __init__(self):
        juice.Juice.__init__(self, True)

    def command_ADD_USER(self, name, password):
        # all security is transport security
        theDomain = self.transport.getQ2QHost().domain
        self.factory.store.addUser(theDomain, name, password)
        return dict()

    command_ADD_USER.command = AddUser

class IdentityAdminFactory:
    def __init__(self, certstore):
        self.store = certstore

    def buildProtocol(self, addr):
        p = IdentityAdmin()
        p.factory = self
        return p

    def examineRequest(self, fromAddress, toAddress, protocolName):
        if toAddress.resource == "accounts" and protocolName == "identity-admin":
            return [(self, "identity admin")]
        return []

class DirectoryCertificateAndUserStore(q2q.DirectoryCertificateStore):
    def __init__(self, filepath):
        q2q.DirectoryCertificateStore.__init__(self, filepath)
        self.userdir = os.path.join(filepath, "users")

    def getPrivateCertificate(self, domain):
        try:
            return q2q.DirectoryCertificateStore.getPrivateCertificate(self, domain)
        except KeyError:
            if len(self.localStore.keys()) > 10:
                # avoid DoS; nobody is going to need autocreated certs for more
                # than 10 domains
                raise
            self.addPrivateCertificate(domain)
        return q2q.DirectoryCertificateStore.getPrivateCertificate(self, domain)



    def addUser(self, domain, username, password):
        domainpath = os.path.join(self.userdir, domain)
        if not os.path.exists(domainpath):
            os.makedirs(domainpath)
        userpath = os.path.join(domainpath, username+".info")
        if os.path.exists(userpath):
            raise NotAllowed()
        f = open(userpath, 'w')
        f.write(juice.Box(username=username,
                          password=md5.md5(password).hexdigest()).serialize())
        f.close()

    def checkUser(self, domain, username, password):
        domainpath = os.path.join(self.userdir, domain)
        if os.path.exists(domainpath):
            filepath = os.path.join(domainpath, username+".info")
            if os.path.exists(filepath):
                data = juice.parseString(open(filepath).read())[0]
                if data['password'] == md5.md5(password).hexdigest():
                    return defer.succeed(True)
        return defer.fail(KeyError("404'd!"))


class StandaloneQ2Q(DependencyService):
    def setup_Q2Q(self, path,
                  q2qPortnum=q2q.port,
                  inboundTCPPortnum=q2q.port+1,
                  publicIP=None
                  ):
        """Set up a Q2Q service.
        """
        store = DirectoryCertificateAndUserStore(path)
        # store.addPrivateCertificate("kazekage")
        # store.addUser("kazekage", "username", "password1234")
        iaf = IdentityAdminFactory(store)

        self.attach(q2q.Q2QService(
                protocolFactoryFactory=IdentityAdminFactory(store).examineRequest,
                certificateStorage=store,
                q2qPortnum=q2qPortnum,
                inboundTCPPortnum=inboundTCPPortnum,
                publicIP=publicIP,
                ))

def defaultConfig():
    # Put this into a .tac file< and customize to your heart's content
    c = Conf()
    s = c.section
    s('q2q',
      path='q2q-data')
    application = deploy(**c)
    return application

deploy = StandaloneQ2Q.deploy



# -*- test-case-name: vertex.test.test_ptcp -*-

import itertools, random, os

from twisted.internet import reactor, protocol, defer, task, error
from twisted.trial import unittest, assertions

from vertex import ptcp

def reallyLossy(method):
    r = random.Random()
    r.seed(42)
    def worseMethod(*a, **kw):
        if r.choice([True, True, False]):
            method(*a, **kw)
    return worseMethod

def insufficientTransmitter(method,  mtu):
    def worseMethod(bytes, addr):
        method(bytes[:mtu], addr)
    return worseMethod


class TestProtocol(protocol.Protocol):
    buffer = None
    def __init__(self):
        self.onConnect = defer.Deferred()
        self.onDisconn = defer.Deferred()
        self._waiting = None
        self.buffer = []

    def connectionMade(self):
        self.onConnect.callback(None)

    def connectionLost(self, reason):
        self.onDisconn.callback(None)

    def gotBytes(self, bytes):
        assert self._waiting is None
        if ''.join(self.buffer) == bytes:
            return defer.succeed(None)
        self._waiting = (defer.Deferred(), bytes)
        return self._waiting[0]

    def dataReceived(self, bytes):
        self.buffer.append(bytes)
        if self._waiting is not None:
            bytes = ''.join(self.buffer)
            if not self._waiting[1].startswith(bytes):
                x = len(os.path.commonprefix([bytes, self._waiting[1]]))
                print x
                print 'it goes wrong starting with', repr(bytes[x:x+100]), repr(self._waiting[1][x:x+100])
            if bytes == self._waiting[1]:
                self._waiting[0].callback(None)
                self._waiting = None

class Django(protocol.ClientFactory):
    def __init__(self):
        self.onConnect = defer.Deferred()

    def buildProtocol(self, addr):
        p = protocol.ClientFactory.buildProtocol(self, addr)
        self.onConnect.callback(p)
        return p

    def clientConnectionFailed(self, conn, err):
        self.onConnect.errback(err)

class ConnectedPTCPMixin:
    serverPort = None

    def setUpForATest(self,
                      ServerProtocol=TestProtocol, ClientProtocol=TestProtocol):
        serverProto = ServerProtocol()
        clientProto = ClientProtocol()


        self.serverProto = serverProto
        self.clientProto = clientProto

        sf = protocol.ServerFactory()
        sf.protocol = lambda: serverProto

        cf = Django()
        cf.protocol = lambda: clientProto

        serverTransport = ptcp.PTCP(sf)
        clientTransport = ptcp.PTCP(None)

        self.serverTransport = serverTransport
        self.clientTransport = clientTransport

        serverPort = reactor.listenUDP(0, serverTransport)
        clientPort = reactor.listenUDP(0, clientTransport)

        self.clientPort = clientPort
        self.serverPort = serverPort

        return (
            serverProto, clientProto,
            sf, cf,
            serverTransport, clientTransport,
            serverPort, clientPort
            )

    def tearDown(self):
        td = []

        for ptcp in (self.serverTransport, self.clientTransport):
            td.append(ptcp.waitForAllConnectionsToClose())
        d = defer.DeferredList(td)
        return d


class TestProducerProtocol(protocol.Protocol):
    NUM_WRITES = 32
    WRITE_SIZE = 32

    def __init__(self):
        self.onConnect = defer.Deferred()
        self.onPaused = defer.Deferred()

    def connectionMade(self):
        self.onConnect.callback(None)
        self.count = -1
        self.transport.registerProducer(self, False)

    def pauseProducing(self):
        if self.onPaused is not None:
            self.onPaused.callback(None)
            self.onPaused = None

    def resumeProducing(self):
        self.count += 1
        if self.count < self.NUM_WRITES:
            bytes = chr(self.count) * self.WRITE_SIZE
            # print 'Issuing a write', len(bytes)
            self.transport.write(bytes)
            if self.count == self.NUM_WRITES - 1:
                # Last time through, intentionally drop the connection before
                # the buffer is empty to ensure we handle this case properly.
                # print 'Disconnecting'
                self.transport.loseConnection()
        else:
            # print 'Unregistering'
            self.transport.unregisterProducer()

class PTCPTransportTestCase(ConnectedPTCPMixin, unittest.TestCase):
    def setUpClass(self):
        ptcp.PTCPConnection._retransmitTimeout /= 10
        ptcp.PTCPPacket.retransmitCount *= 10

    def tearDownClass(self):
        ptcp.PTCPConnection._retransmitTimeout *= 10
        ptcp.PTCPPacket.retransmitCount /= 10

    def xtestWhoAmI(self):
        (serverProto, clientProto,
         sf, cf,
         serverTransport, clientTransport,
         serverPort, clientPort) = self.setUpForATest()

        def gotAddress(results):
            (serverSuccess, serverAddress), (clientSuccess, clientAddress) = results
            self.failUnless(serverSuccess)
            self.failUnless(clientSuccess)

            self.assertEquals(serverAddress[1], serverPort.getHost().port)
            self.assertEquals(clientAddress[1], clientPort.getHost().port)

        def connectionsMade(ignored):
            return defer.DeferredList([serverProto.transport.whoami(), clientProto.transport.whoami()]).addCallback(gotAddress)

        clientConnID = clientTransport.connect(cf, '127.0.0.1', serverPort.getHost().port)

        return defer.DeferredList([serverProto.onConnect, clientProto.onConnect]).addCallback(connectionsMade)

    #testWhoAmI.skip = 'arglebargle'

    def testVerySimpleConnection(self):
        (serverProto, clientProto,
         sf, cf,
         serverTransport, clientTransport,
         serverPort, clientPort) = self.setUpForATest()


        clientConnID = clientTransport.connect(cf, '127.0.0.1', serverPort.getHost().port)

        def sendSomeBytes(ignored, n=10, server=False):
            if n:
                bytes = 'not a lot of bytes' * 1000
                if server:
                    serverProto.transport.write(bytes)
                else:
                    clientProto.transport.write(bytes)
                if server:
                    clientProto.buffer = []
                    d = clientProto.gotBytes(bytes)
                else:
                    serverProto.buffer = []
                    d = serverProto.gotBytes(bytes)
                return d.addCallback(sendSomeBytes, n - 1, not server)

        def loseConnections(ignored):
            serverProto.transport.loseConnection()
            clientProto.transport.loseConnection()
            return defer.DeferredList([
                    serverProto.onDisconn,
                    clientProto.onDisconn
                    ])

        dl = defer.DeferredList([serverProto.onConnect, clientProto.onConnect])
        dl.addCallback(sendSomeBytes)
        dl.addCallback(loseConnections)
        return dl


    def testProducerConsumer(self):
        (serverProto, clientProto,
         sf, cf,
         serverTransport, clientTransport,
         serverPort, clientPort) = self.setUpForATest(
            ServerProtocol=TestProducerProtocol)

        def disconnected(ignored):
            self.assertEquals(
                ''.join(clientProto.buffer),
                ''.join([chr(n) * serverProto.WRITE_SIZE
                         for n in range(serverProto.NUM_WRITES)]))

        clientConnID = clientTransport.connect(cf, '127.0.0.1', serverPort.getHost().port)
        return clientProto.onDisconn.addCallback(disconnected)


    def testTransportProducer(self):
        (serverProto, clientProto,
         sf, cf,
         serverTransport, clientTransport,
         serverPort, clientPort) = self.setUpForATest()

        resumed = []
        def resumeProducing():
            resumed.append(True)
            clientProto.transport.resumeProducing()

        def cbBytes(ignored):
            self.failUnless(resumed)
            clientProto.transport.loseConnection()

        def cbConnect(ignored):
            BYTES = 'Here are bytes'
            clientProto.transport.pauseProducing()
            serverProto.transport.write(BYTES)
            reactor.callLater(2, resumeProducing)
            return clientProto.gotBytes(BYTES).addCallback(cbBytes)


        clientConnID = clientTransport.connect(cf, '127.0.0.1', serverPort.getHost().port)
        connD = defer.DeferredList([clientProto.onConnect, serverProto.onConnect])
        connD.addCallback(cbConnect)
        return connD

    def testTransportProducerProtocolProducer(self):
        (serverProto, clientProto,
         sf, cf,
         serverTransport, clientTransport,
         serverPort, clientPort) = self.setUpForATest(
            ServerProtocol=TestProducerProtocol)

        paused = []
        def cbPaused(ignored):
            # print 'Paused'
            paused.append(True)
            # print 'RESUMING', clientProto, clientTransport, clientPort
            clientProto.transport.resumeProducing()
        serverProto.onPaused.addCallback(cbPaused)

        def cbBytes(ignored):
            # print 'Disconnected'
            self.assertEquals(
                ''.join(clientProto.buffer),
                ''.join([chr(n) * serverProto.WRITE_SIZE
                         for n in range(serverProto.NUM_WRITES)]))

        def cbConnect(ignored):
            # The server must write enough to completely fill the outgoing buffer,
            # since our peer isn't ACKing /anything/ and our server waits for
            # writes to be acked before proceeding.
            serverProto.WRITE_SIZE = serverProto.transport.sendWindow * 5

            # print 'Connected'
            # print 'PAUSING CLIENT PROTO', clientProto, clientTransport, clientPort
            clientProto.transport.pauseProducing()
            return clientProto.onDisconn.addCallback(cbBytes)

        clientConnID = clientTransport.connect(cf, '127.0.0.1', serverPort.getHost().port)
        connD = defer.DeferredList([clientProto.onConnect, serverProto.onConnect])
        connD.addCallback(cbConnect)
        return connD


class LossyTransportTestCase(PTCPTransportTestCase):
    def setUpForATest(self, *a, **kw):
        results = PTCPTransportTestCase.setUpForATest(self, *a, **kw)
        results[-2].write = reallyLossy(results[-2].write)
        results[-1].write = reallyLossy(results[-1].write)
        return results


class SmallMTUTransportTestCase(PTCPTransportTestCase):
    def setUpForATest(self, *a, **kw):
        results = PTCPTransportTestCase.setUpForATest(self, *a, **kw)
        results[-2].write = insufficientTransmitter(results[-2].write, 128)
        results[-1].write = insufficientTransmitter(results[-1].write, 128)
        return results

class TimeoutTestCase(ConnectedPTCPMixin, unittest.TestCase):
    def setUpClass(self):
        ptcp.PTCPConnection._retransmitTimeout /= 10

    def tearDownClass(self):
        ptcp.PTCPConnection._retransmitTimeout *= 10

    def testConnectTimeout(self):
        (serverProto, clientProto,
         sf, cf,
         serverTransport, clientTransport,
         serverPort, clientPort) = self.setUpForATest()

        clientTransport.sendPacket = lambda *a, **kw: None
        clientConnID = clientTransport.connect(cf, '127.0.0.1', serverPort.getHost().port)
        return assertions.assertFailure(cf.onConnect, error.TimeoutError)

    def testDataTimeout(self):
        (serverProto, clientProto,
         sf, cf,
         serverTransport, clientTransport,
         serverPort, clientPort) = self.setUpForATest()

        def cbConnected(ignored):
            serverProto.transport.ptcp.sendPacket = lambda *a, **kw: None
            clientProto.transport.write('Receive this data.')
            serverProto.transport.write('Send this data.') # have to send data
                                                           # or the server will
                                                           # never time out:
                                                           # need a
                                                           # SO_KEEPALIVE
                                                           # option somewhere
            return clientProto.onDisconn

        clientConnID = clientTransport.connect(cf, '127.0.0.1', serverPort.getHost().port)

        d = defer.DeferredList([serverProto.onConnect, clientProto.onConnect])
        d.addCallback(cbConnected)
        return d
